@@ -26,6 +26,9 @@
 
 #include <OBULLET.h>
 
+struct BulletHomingCrc;
+struct BulletHomingGF;
+
 #pragma pack(1)
 class BulletHoming : public Bullet
 {
@@ -44,12 +47,15 @@ public:
 
 	void	process_move();
 
-	// int	write_derived_file(File *);
-	// int	read_derived_file(File *);
+	int	write_derived_file(File *filePtr);
+	int	read_derived_file(File *filePtr);
+	void	write_derived_record(BulletHomingGF *r);
+	void	read_derived_record(BulletHomingGF *r);
 
 	//-------------- multiplayer checking codes ---------------//
-	virtual	UCHAR crc8();
+	virtual	uint8_t crc8();
 	virtual	void	clear_ptr();
+	virtual	void	init_crc(BulletHomingCrc *c);
 };
 #pragma pack()
 
