@@ -34,12 +34,15 @@
 
 //------- Define class FirmWar --------//
 
+struct FirmWarCrc;
+struct FirmWarGF;
+
 #pragma pack(1)
 class FirmWar : public Firm
 {
 public:
 	short build_unit_id;
-	DWORD last_process_build_frame_no;
+	uint32_t last_process_build_frame_no;
 	float build_progress_days;
 
 	char  build_queue_array[MAX_BUILD_QUEUE];		// it stores the unit id.
@@ -52,13 +55,13 @@ public:
 	~FirmWar();
 
 	void 	put_info(int refreshFlag);
-	void 	detect_info();
+	int	detect_info();
 
 	void 	disp_main_menu(int refreshFlag);
-	void 	detect_main_menu();
+	int	detect_main_menu();
 
 	void 	disp_build_menu(int refreshFlag);
-	void 	detect_build_menu();
+	int	detect_build_menu();
 
 	void    change_nation(int newNationRecno);
 
@@ -71,8 +74,14 @@ public:
 	void	cancel_build_unit();
 
 	//-------------- multiplayer checking codes ---------------//
-	virtual	UCHAR	crc8();
+	virtual	uint8_t	crc8();
 	virtual	void	clear_ptr();
+	virtual	void	init_crc(FirmWarCrc *c);
+
+	int	write_derived_file(File *filePtr);
+	int	read_derived_file(File *filePtr);
+	void	write_derived_record(FirmWarGF *r);
+	void	read_derived_record(FirmWarGF *r);
 
 	enum {FIRMWAR_BUILD_BATCH_COUNT = 10}; // Number of units enqueued when holding shift - ensure this is less than MAX_BUILD_QUEUE
 
