@@ -188,6 +188,8 @@ enum  {  KEEP_PRESERVE_ACTION = 1,  // used for stop2() to keep preserve action
 
 //----------- Define TeamInfo -------------//
 
+struct TeamInfoGF;
+
 #pragma pack(1)
 struct TeamInfo
 {
@@ -196,11 +198,18 @@ struct TeamInfo
    char  member_count;
    short member_unit_array[MAX_TEAM_MEMBER];
    int   ai_last_request_defense_date;
+
+	void write_record(TeamInfoGF *r);
+	void read_record(TeamInfoGF *r);
 };
 #pragma pack()
 
+
 //----------- Define class Unit -----------//
 
+struct UnitCrc;
+struct UnitGF;
+
 #pragma pack(1)
 class Unit : public Sprite
 {
@@ -210,12 +219,12 @@ public:
 	char        race_id;
 	char        nation_recno;
 	char        ai_unit;
-	WORD        name_id;          // id. of the unit's name in RaceRes::first_name_array;
+	uint16_t    name_id;             // id. of the unit's name in RaceRes::first_name_array;
 
-	DWORD       unit_group_id;       // the group id this unit belong to if it is selected
-	DWORD       team_id;             // id. of defined team
+	uint32_t       unit_group_id;       // the group id this unit belong to if it is selected
+	uint32_t       team_id;             // id. of defined team
 	char        selected_flag;       // whether the unit has been selected or not
-	char			group_select_id;		// id for group selection
+	char        group_select_id;     // id for group selection
 
 	char        waiting_term;        // for 2x2 unit only, the term to wait before recalling A* to get a new path
 	char        blocked_by_member;
@@ -225,7 +234,7 @@ public:
 
 	int           is_visible()      { return cur_x >= 0; }     // whether the unit is visible on the map, it is not invisable if cur_x == -1
 	virtual char* unit_name(int withTitle=1);
-	BYTE          region_id();
+	uint8_t          region_id();
 
 	//--------- action vars ------------//
 	char        action_misc;
@@ -242,7 +251,7 @@ public:
 	short       action_y_loc2;
 
 	char        blocked_edge[4];        // for calling searching in attacking
-	UCHAR       attack_dir;
+	uint8_t       attack_dir;
 
 	//------------ attack parameters -----------//
 
@@ -310,7 +319,7 @@ public:
 
 	//--------- AI parameters ------------//
 
-	WORD        ai_action_id;     			// an unique id. for locating the AI action node this unit belongs to in Nation::action_array
+	uint16_t        ai_action_id;     			// an unique id. for locating the AI action node this unit belongs to in Nation::action_array
 
 	char  		original_action_mode;
 	short			original_action_para;
@@ -370,7 +379,7 @@ public:
 			  void draw_skill_icon();
 
 			  void set_spy(int spyRecno);
-			  void set_name(WORD newNameId);
+			  void set_name(uint16_t newNameId);
 			  void set_mode(char modeId, short modePara=0) { unit_mode=modeId; unit_mode_para=modePara; }
 			  int  is_shealth();
 			  int  is_civilian();
@@ -380,6 +389,8 @@ public:
 			  int  true_nation_recno();            // the true nation recno of the unit, taking care of the situation where the unit is a spy
 	virtual int  is_ai_all_stop();
 			  int  get_cur_loc(short& xLoc, short& yLoc);
+			  int  get_cur_loc2(short& xLoc, short& yLoc);
+			  short is_leader_in_range();
 
 	virtual void die()         {;}
 
@@ -474,11 +485,11 @@ public:
 	void  attack_town(int townXLoc, int townYLoc, int xOffset=0, int yOffset=0, int resetBlockedEdge=1);
 	void  attack_wall(int wallXLoc, int wallYLoc, int xOffset=0, int yOffset=0, int resetBlockedEdge=1);
 
-	void  hit_target(Unit* parentUnit, Unit* targetUnit, float attackDamage);
-	void  hit_building(Unit* parentUnit, int targetXLoc, int targetYLoc, float attackDamage);
-	void  hit_firm(Unit* parentUnit, int targetXLoc, int targetYLoc, float attackDamage);
-	void  hit_town(Unit* parentUnit, int targetXLoc, int targetYLoc, float attackDamage);
-	void  hit_wall(Unit* attackUnit, int targetXLoc, int targetYLoc, float attackDamage);
+	void  hit_target(Unit* parentUnit, Unit* targetUnit, float attackDamage, short parentNationRecno);
+	void  hit_building(Unit* attackUnit, int targetXLoc, int targetYLoc, float attackDamage, short attackNationRecno);
+	void  hit_firm(Unit* attackUnit, int targetXLoc, int targetYLoc, float attackDamage, short attackNationRecno);
+	void  hit_town(Unit* attackUnit, int targetXLoc, int targetYLoc, float attackDamage, short attackNationRecno);
+	void  hit_wall(Unit* attackUnit, int targetXLoc, int targetYLoc, float attackDamage, short attackNationRecno);
 
 	int   max_attack_range();
 	void  gain_experience();
@@ -560,14 +571,17 @@ public:
 
 	int   write_file(File* filePtr);
 	int   read_file(File* filePtr);
+	void  write_record(UnitGF* r);
+	void  read_record(UnitGF* r);
 
 	virtual int write_derived_file(File* filePtr);
 	virtual int read_derived_file(File* filePtr);
 	virtual void fix_attack_info();         // set attack_info_array appropriately
 
 	//-------------- multiplayer checking codes ---------------//
-	virtual	UCHAR crc8();
+	virtual	uint8_t crc8();
 	virtual	void	clear_ptr();
+	virtual	void	init_crc(UnitCrc *c);
 
 private:
 	//------------------ idle functions -------------------//
@@ -634,7 +648,7 @@ private:
 	int   space_for_attack(int targetXLoc, int targetYLoc, char targetMobileType, int targetWidth, int targetHeight);
 	int   space_around_target(int squareXLoc, int squareYLoc, int width, int height);
 	int   space_around_target_ver2(int targetXLoc, int targetYLoc, int targetWidth, int targetHeight);
-	int   ship_surr_has_free_land(int targetXLoc, int targetYLoc, UCHAR regionId);
+	int   ship_surr_has_free_land(int targetXLoc, int targetYLoc, uint8_t regionId);
 	int   free_space_for_range_attack(int targetXLoc, int targetYLoc, int targetWidth, int targetHeight, int targetMobileType, int maxRange);
 
 	void  choose_best_attack_mode(int attackDistance, char targetMobileType=UNIT_LAND);
@@ -681,7 +695,7 @@ private:
 	int   monster_defend_follow_target();
 
 	//---------- embark to ship and other ship functions ---------//
-	int   ship_to_beach_path_edit(int& resultXLoc, int& resultYLoc, UCHAR regionNo);
+	int   ship_to_beach_path_edit(int& resultXLoc, int& resultYLoc, uint8_t regionId);
 	void  ship_leave_beach(int shipOldXLoc, int shipOldYLoc);
 
 	//---------------- other functions -----------------//
@@ -708,6 +722,7 @@ protected:
 	void  detect_spy_menu(int dispY1);
 	int   spy_menu_height();
 	void  disp_hit_point(int dispY1);
+	int   detect_select_hotkey();
 
 	void  process_attack_unit();
 	void  process_attack_firm();
@@ -744,16 +759,12 @@ public:
 	short selected_recno;
 	short selected_count;
 
-	DWORD cur_group_id;            // for Unit::unit_group_id
-	DWORD cur_team_id;             // for Unit::team_id
+	uint32_t cur_group_id;            // for Unit::unit_group_id
+	uint32_t cur_team_id;             // for Unit::team_id
 
 	short idle_blocked_unit_reset_count; // used to improve performance for searching related to attack
 
 	short visible_unit_count;
-	char	mp_first_frame_to_select_caravan;	// for multiplayer, true if 1st frame to select caravan
-	char	mp_first_frame_to_select_ship;		// ditto for ship
-	short	mp_pre_selected_caravan_recno;		// for multiplayer, 0 or recno that caravan selected in previous frame
-	short	mp_pre_selected_ship_recno;			// ditto for ship
 
 	static short   selected_land_unit_count;
 	static short   selected_sea_unit_count;
@@ -812,21 +823,6 @@ public:
 	//--------- unit filter function ----------//
 	int   divide_attack_by_nation(short nationRecno, short *selectedArray, int selectedCount);
 
-	//--------- for multiplayers' caravan and ship ---------//
-	void	mp_mark_selected_caravan();
-	int	mp_get_selected_caravan_count();
-	void	mp_reset_selected_caravan_count();
-	void	mp_add_selected_caravan(short unitRecno);
-	int	mp_is_selected_caravan(short unitRecno);
-
-	void	mp_mark_selected_ship();
-	int	mp_get_selected_ship_count();
-	void	mp_reset_selected_ship_count();
-	void	mp_add_selected_ship(short unitRecno);
-	int	mp_is_selected_ship(short unitRecno);
-
-	void	update_selected_trade_unit_info();
-
 	int   write_file(File* filePtr);
 	int   read_file(File* filePtr);
 
@@ -839,6 +835,8 @@ public:
 	int   is_deleted(int recNo);
 	int   is_truly_deleted(int recNo);
 
+	void  disp_next(int seekDir, int sameNation);
+
 private:
 	void  divide_array(int locX, int locY, short* selectedArray, int selectedCount, int excludeSelectedLocUnit=0);
 	void  set_group_id(short* selectedArray, int selectedCount);
@@ -858,7 +856,7 @@ private:
 	char* get_target_x_offset(int targetWidth, int targetHeight, char curDir);
 	char* get_target_y_offset(int targetWidth, int targetHeight, char curDir);
 
-	void  arrange_units_in_group(int xLoc1, int yLoc1, int xLoc2, int yLoc2, short* selectedUnitArray, int selectedCount, DWORD unitGroupId, int targetType);
+	void  arrange_units_in_group(int xLoc1, int yLoc1, int xLoc2, int yLoc2, short* selectedUnitArray, int selectedCount, uint32_t unitGroupId, int targetType);
 	int   analyse_surround_location(int targetXLoc, int targetYLoc, int targetWidth, int targetHeight, char mobileType);
 	void  check_nearby_location(int targetXLoc, int targetYLoc, char xOffset, char yOffset, int targetWidth, int targetHeight, char targetMobileType, int& analyzeResult);
 	void  handle_attack_target_totally_blocked(int targetXLoc, int targetYLoc, short targetRecno, short *selectedUnitArray, short selectedCount, int targetType);
