@@ -26,6 +26,9 @@
 
 #include <OUNIT.h>
 
+struct UnitExpCartCrc;
+struct UnitExpCartGF;
+
 #pragma pack(1)
 class UnitExpCart : public Unit
 {
@@ -40,8 +43,14 @@ public:
 	void	trigger_explode();
 
 	//-------------- multiplayer checking codes ---------------//
-	virtual	UCHAR crc8();
+	virtual	uint8_t crc8();
 	virtual	void	clear_ptr();
+	virtual	void	init_crc(UnitExpCartCrc *c);
+
+	int	write_derived_file(File *filePtr);
+	int	read_derived_file(File *filePtr);
+	void	write_derived_record(UnitExpCartGF *r);
+	void	read_derived_record(UnitExpCartGF *r);
 };
 #pragma pack()
 
