@@ -33,10 +33,12 @@
 #include <OBUTTON.h>
 #include <OBATTLE.h>
 #include <OGFILE.h>
+#include <OGAMHALL.h>
 #include <OMUSIC.h>
 #include <OGAME.h>
 #include <OVGALOCK.h>
 #include "gettext.h"
+#include <ConfigAdv.h>
 
 #ifdef DEMO
 #define DISABLE_MULTI_PLAYER
@@ -258,6 +260,13 @@ void Game::main_menu()
 			}
 		}
 
+		// method to start a game replay at this time
+		if( mouse.is_key_event() && mouse.scan_code == 'r')
+		{
+			battle.run_replay();
+			refreshFlag=1;
+		}
+
 		//-------------------------------------//
 
 		if( sys.signal_exit_flag == 1 || i == MAIN_OPTION_COUNT-1 )			// quit the system now
@@ -319,7 +328,7 @@ void Game::run_main_menu_option(int optionId)
 
 	if( optionId==4 )
 	{
-		game_file_array.disp_hall_of_fame();
+		hall_of_fame.disp_hall_of_fame();
 	}
 
 	//------------- Credits -----------//
@@ -366,6 +375,10 @@ void Game::disp_version()
 		str += "-dev";
 	#endif
 
+	#ifndef HAVE_KNOWN_BUILD
+		str += "?";
+	#endif
+
 	#ifdef DEBUG
 		str += " (DEBUG)";
 	#endif
@@ -608,10 +621,11 @@ void Game::single_player_menu()
 						break;
 
 					case 3:
-						game_file_array.init("*.SAV");
+						game_file_array.init(glob_save_file);
 
-						if( game_file_array.menu(2) == 1)
+						if( game_file_array.load_game() == 1)
 						{
+							sys.set_speed(config_adv.game_load_default_frame_speed, COMMAND_AUTO);
 							battle.run_loaded();
 							deinit();
 						}
@@ -849,7 +863,7 @@ void Game::multi_player_menu(int lobbied, char *game_host)
 						// ##### begin Gilbert 26/8 ######//
 						{
 							int loadedRecno = 0;
-							game_file_array.init("*.SVM");
+							game_file_array.init(glob_save_multi_file);
 							if( game_file_array.menu(2, &loadedRecno) == 1 )
 							{
 								err_when( !loadedRecno );
