@@ -19,14 +19,15 @@
  */
 
 //Filename    : OGFILEA.CPP
-//Description : Game File Array
+//Description : Game File Array / UI.
 
-#ifndef NO_WINDOWS
+#ifdef USE_WINDOWS
 #include <io.h>
-#else
+#elif defined USE_POSIX
 #include <unistd.h>
+#include <time.h>
 #endif
-
+#include <OGFILE.h>
 #include <KEY.h>
 #include <ODIR.h>
 #include <OSYS.h>
@@ -45,17 +46,35 @@
 #include <OGAME.h>
 #include <OGAMESET.h>
 #include <OGFILE.h>
+#include <OGAMHALL.h>
 #include <OBUTT3D.h>
 #include <OSLIDCUS.h>
 #include <OBLOB.h>
 #include <dbglog.h>
+#include <ONATIONA.h>
 #include "gettext.h"
 
-DBGLOG_DEFAULT_CHANNEL(GameFile);
+#include <string.h> //strncpy
+#include <posix_string_compat.h> //strnicmp
+
+#ifdef USE_WINDOWS
+#include <windows.h>
+#endif
+
 
 
 //--------- Define constant ---------//
 
+#ifndef GLOB_SAVE_FILE
+#define GLOB_SAVE_FILE "*.SAV"
+#endif
+#ifndef GLOB_SAVE_MULTI_FILE
+#define GLOB_SAVE_MULTI_FILE "*.SVM"
+#endif
+
+const char* glob_save_file = GLOB_SAVE_FILE;
+const char* glob_save_multi_file = GLOB_SAVE_MULTI_FILE;
+
 enum { FILE_MENU_WIDTH = 638,
 		 FILE_MENU_HEIGHT = 398 };
 
@@ -82,7 +101,6 @@ enum { SCROLL_X1 = 595,
 
 #define MAX_BROWSE_DISP_REC	   5		// MAX. no. of records can be displayed in the saved game browser
 
-#define HALL_OF_FAME_FILE_NAME  "HALLFAME.DAT"
 
 //------- Declare static vars & functions ----------//
 
@@ -92,9 +110,8 @@ static short	browse_top_recno;
 static short	menu_x1, menu_y1;
 
 static int     sort_game_file_function( const void *a, const void *b );
-static int     last_game_recno();
-static void    key_search();
-static void		disp_scroll_bar_func(SlideVBar *scroll, int);
+static void    disp_scroll_bar_func(SlideVBar *scroll, int);
+
 
 //------ Begin of function GameFileArray constuctor ------//
 
@@ -110,10 +127,8 @@ GameFileArray::GameFileArray() : DynArray( sizeof(GameFile), 10 )
 		demo_format = 1;
 	#endif
 
-	last_file_name[0] = '\0';
 	has_read_hall_of_fame = 0;
-
-	memset( hall_fame_array, 0, sizeof(HallFame)*HALL_FAME_NUM );
+	last_file_name[0] = '\0';
 }
 //-------- End of function GameFileArray constuctor ------//
 
@@ -122,11 +137,10 @@ GameFileArray::GameFileArray() : DynArray( sizeof(GameFile), 10 )
 
 void GameFileArray::init(const char *extStr)
 {
-	//------------- Read Hall of Fame ------------//
-
 	if( !has_read_hall_of_fame )		// only read once, GameFileArray::init() is called every time the load/save game menu is brought up.
 	{
-		read_hall_of_fame();
+		strncpy(last_file_name, hall_of_fame.get_last_savegame_file_name(), FilePath::MAX_FILE_PATH);
+		last_file_name[FilePath::MAX_FILE_PATH] = '\0';
 		has_read_hall_of_fame = 1;
 	}
 
@@ -141,8 +155,8 @@ void GameFileArray::init(const char *extStr)
 
 void GameFileArray::deinit()
 {
-	//-- Need to save hall of fame, because it contains last loaded/saved game --//
-	write_hall_of_fame();
+	//-- Need to transfer last savegame filename to hall of fame, because it contains last loaded/saved game --//
+	hall_of_fame.set_last_savegame_file_name(last_file_name);
 }
 //-------- End of function GameFileArray::deinit ------//
 
@@ -161,11 +175,11 @@ void GameFileArray::deinit()
 //                    1 - save game
 //                    2 - load game
 //
-// <int *> recno    = if overwritting save game or load game acion 
+// <int *> recno    = if overwritting save game or load game action
 //                    is succcessful, return the recno of GameFile
 //
 // return : <int> 1 - game loaded/saved
-//                0 - user cancel loading/saving 
+//                0 - user cancel loading/saving
 //               -1 - loading/saving error
 //
 int GameFileArray::menu(int actionMode, int *recno)
@@ -205,13 +219,6 @@ int GameFileArray::menu(int actionMode, int *recno)
 		return 1;
 	}
 
-	// #### begin Gilbert 25/9 ########//
-	//#ifdef DEMO          // No game saving in demo version
-	//	box.msg( "Sorry, you cannot load/save game in the demo version." );
-	//	return -1;
-	//#endif
-	// #### end Gilbert 25/9 ########//
-
 	action_mode = actionMode;
 
 	if( action_mode==2 && size()==0 )
@@ -269,7 +276,7 @@ int GameFileArray::menu(int actionMode, int *recno)
 
 	for( int i=1 ; i<=size() ; i++ )
 	{
-		if( strcmp(last_file_name, game_file_array[i]->file_name)==0 )
+		if( strcmp(last_file_name, (*this)[i]->file_name)==0 )
 		{
 			browse_recno = i;
 			break;
@@ -351,14 +358,14 @@ int GameFileArray::menu(int actionMode, int *recno)
 				for( int j = 0; j < MAX_BROWSE_DISP_REC; ++j)
 				{
 					browseArea[j].resize(2*sizeof(short)+BROWSE_REC_WIDTH*BROWSE_REC_HEIGHT);
-					vga_front.read_bitmap( 
+					vga_front.read_bitmap(
 						menu_x1+BROWSE_X1, menu_y1+BROWSE_Y1+j*BROWSE_REC_HEIGHT,
 						menu_x1+BROWSE_X2, menu_y1+BROWSE_Y1+j*BROWSE_REC_HEIGHT+BROWSE_REC_HEIGHT-1,
 						browseArea[j].ptr);
 				}
 
 				scrollArea.resize(2*sizeof(short)+SCROLL_WIDTH*SCROLL_HEIGHT);
-				vga_front.read_bitmap( menu_x1+SCROLL_X1, menu_y1+SCROLL_Y1, 
+				vga_front.read_bitmap( menu_x1+SCROLL_X1, menu_y1+SCROLL_Y1,
 					menu_x1+SCROLL_X2, menu_y1+SCROLL_Y2, scrollArea.ptr);
 
 				mouse.show_area();
@@ -406,7 +413,7 @@ int GameFileArray::menu(int actionMode, int *recno)
 						}
 						else if( rec <= size() )
 						{
-							game_file_array[rec]->disp_info(browseSlotX1, browseSlotY1);
+							disp_entry_info((*this)[rec], browseSlotX1, browseSlotY1);
 						}
 						if( rec == browse_recno )
 						{
@@ -448,8 +455,8 @@ int GameFileArray::menu(int actionMode, int *recno)
 				refreshFlag |= LSOPTION_SCROLL | LSOPTION_ALL_SLOTS;
 			browse_top_recno = scrollBar.view_recno;
 		}
-		else if( mouse.double_click( menu_x1+BROWSE_X1, menu_y1+BROWSE_Y1, 
-			menu_x1+BROWSE_X1+BROWSE_REC_WIDTH-1, 
+		else if( mouse.double_click( menu_x1+BROWSE_X1, menu_y1+BROWSE_Y1,
+			menu_x1+BROWSE_X1+BROWSE_REC_WIDTH-1,
 			menu_y1+BROWSE_Y1+ BROWSE_REC_HEIGHT*MAX_BROWSE_DISP_REC -1) )
 		{
 			// double click on game slot
@@ -474,8 +481,8 @@ int GameFileArray::menu(int actionMode, int *recno)
 				// ######## end Gilbert 31/10 ########//
 			}
 		}
-		else if( mouse.single_click( menu_x1+BROWSE_X1, menu_y1+BROWSE_Y1, 
-			menu_x1+BROWSE_X1+BROWSE_REC_WIDTH-1, 
+		else if( mouse.single_click( menu_x1+BROWSE_X1, menu_y1+BROWSE_Y1,
+			menu_x1+BROWSE_X1+BROWSE_REC_WIDTH-1,
 			menu_y1+BROWSE_Y1+ BROWSE_REC_HEIGHT*MAX_BROWSE_DISP_REC -1) )
 		{
 			// click on game slot
@@ -596,7 +603,7 @@ void GameFileArray::disp_browse()
 		}
 		else
 		{
-			game_file_array[i]->disp_info(x, y);
+			disp_entry_info((*this)[i], x, y);
 		}
 		if( i == browse_recno )
 		{
@@ -608,17 +615,35 @@ void GameFileArray::disp_browse()
 //--------- End of function GameFileArray::disp_browse --------//
 
 
-//-------- Begin of function GameFile::disp_info --------//
-//
-void GameFile::disp_info(int x, int y)
+static time_t filetime_to_posix(const GameFileDate* gameFileDate)
 {
-	vga_front.put_bitmap(x+10, y+10,	unit_res[ race_res[race_id]->basic_unit_id ]->king_icon_ptr);
+	time_t t;
+
+	t = (uint64_t)gameFileDate->dwHighDateTime << 32;
+	t |= gameFileDate->dwLowDateTime;
+
+	// 100-nanoseconds = milliseconds * 10000
+	// removes the diff between 1970 and 1601
+	t -= 11644473600000 * 10000;
+
+	// converts back from 100-nanoseconds to seconds
+	t /= 10000000;
+
+	return t;
+}
+
+
+//-------- Begin of function GameFileArray::disp_entry_info --------//
+//
+void GameFileArray::disp_entry_info(const GameFile* entry, int x, int y)
+{
+	vga_front.put_bitmap(x+10, y+10,	unit_res[ race_res[entry->race_id]->basic_unit_id ]->king_icon_ptr);
 
 	x+=60;
 
 	//------ display player color -----//
 
-	nation_array.disp_nation_color( x+1, y+13, nation_color );
+	nation_array.disp_nation_color( x+1, y+13, entry->nation_color );
 
 	//-------- display king name ------//
 
@@ -626,21 +651,23 @@ void GameFile::disp_info(int x, int y)
 
 	str  = _("King");
 	str += " ";
-	str += player_name;
+	str += entry->player_name;
 
 	font_bible.put( x+18, y+8, str );
 
 	//------- display game date --------//
 
-	str  = _("Game Date: ");
-	str += date.date_str(game_date);
+	str  = _("Game Date");
+	str += ": ";
+	str += date.date_str(entry->game_date);
 
 	font_bible.put( x, y+30, str );
 
 	//---------------------------------//
 
-	str  = _("File Name: ");
-	str += file_name;
+	str  = _("File Name");
+	str += ": ";
+	str += entry->file_name;
 
 	#if(defined(FRENCH))
 		font_small.put( x+320, y+16, str );
@@ -650,19 +677,24 @@ void GameFile::disp_info(int x, int y)
 		font_small.put( x+335, y+16, str );
 	#endif
 
+	str  = _("File Date");
+	str += ": ";
+#ifdef USE_WINDOWS
 	SYSTEMTIME sysTime;
 	FILETIME localFileTime;
-#ifndef NO_WINDOWS  //FIXME
-	FileTimeToLocalFileTime( &file_date, &localFileTime );
+	FileTimeToLocalFileTime( (FILETIME*)&entry->file_date, &localFileTime );
 	FileTimeToSystemTime( &localFileTime, &sysTime );
-#else
-	memset(&sysTime, 0, sizeof(SYSTEMTIME));
-#endif
 
-	str  = _("File Date: ");
-	str += date.date_str(date.julian(sysTime.wYear, sysTime.wMonth,sysTime.wDay), 1);
+	str += date.date_str(date.julian(sysTime.wYear, sysTime.wMonth, sysTime.wDay), 1);
 	str += " ";
-	str += date.time_str( sysTime.wHour * 100 + sysTime.wMinute );
+	str += date.time_str(sysTime.wHour*100 + sysTime.wMinute);
+#elif defined USE_POSIX
+	time_t gameFileDate = filetime_to_posix(&entry->file_date);
+	struct tm* lt = localtime(&gameFileDate);
+	str += date.date_str(date.julian(lt->tm_year+1900, lt->tm_mon+1, lt->tm_mday), 1);
+	str += " ";
+	str += date.time_str(lt->tm_hour*100 + lt->tm_min);
+#endif
 
 	#if(defined(FRENCH))
 		font_small.put( x+318, y+34, str );
@@ -672,7 +704,7 @@ void GameFile::disp_info(int x, int y)
 		font_small.put( x+335, y+34, str );
 	#endif
 }
-//--------- End of function GameFile::disp_info --------//
+//--------- End of function GameFileArray::disp_entry_info --------//
 
 
 //------- Begin of function GameFileArray::process_action ------//
@@ -697,13 +729,15 @@ int GameFileArray::process_action(int saveNew)
 		}
 		else           // save on existing slot
 		{
-			if( !box.ask( _("It will overwrite the existing saved game, proceed ?") ) )
+			if( !box.ask( _("Overwrite the existing saved game?") ) )
 				return 0;
 
-			GameFile* gameFile = game_file_array[browse_recno];
-
-			if( !gameFile->save_game())
+			GameFile* gameFile = (*this)[browse_recno];
+			if( !gameFile->save_game() )
+			{
+				box.msg(gameFile->status_str());
 				return -1;
+			}
 
 			strcpy( last_file_name, gameFile->file_name );
 		}
@@ -715,11 +749,17 @@ int GameFileArray::process_action(int saveNew)
 
 	else
 	{
-		GameFile* gameFile = game_file_array[browse_recno];
+		GameFile* gameFile = (*this)[browse_recno];
 
 		int rc = gameFile->load_game((const char*)sys.dir_config, NULL);
 		if( rc > 0 )
-			strcpy( last_file_name, gameFile->file_name );
+		{
+			strcpy(last_file_name, (*this)[browse_recno]->file_name);
+		}
+		else
+		{
+			box.msg(gameFile->status_str());
+		}
 		return rc;
 	}
 
@@ -742,20 +782,23 @@ int GameFileArray::process_action(int saveNew)
 //
 int GameFileArray::save_new_game(const char* fileName)
 {
-	GameFile  gameFile;
-	GameFile* gameFilePtr;
-	int       addFlag=1;
-	int       gameFileRecno;
-
-	memset( &gameFile, 0, sizeof(GameFile) );
+	GameFile gameFile;
+	int addFlag=1;
+	int gameFileRecno;
 
 	if( fileName )
 	{
+		if( strlen(fileName) >= FilePath::MAX_FILE_PATH )
+		{
+			box.msg(_("Cannot save the game because the path is too long"));
+			return 0;
+		}
+
 		//----- check for overwriting an existing file ----//
 
-		for( gameFileRecno=1 ; gameFileRecno<=game_file_array.size() ; gameFileRecno++ )
+		for( gameFileRecno=1 ; gameFileRecno<=this->size() ; gameFileRecno++ )
 		{
-			gameFilePtr = game_file_array[gameFileRecno];
+			GameFile* gameFilePtr = (*this)[gameFileRecno];
 
 			if( strcmp(gameFilePtr->file_name, fileName)==0 )      // if this file name already exist
 			{
@@ -777,43 +820,136 @@ int GameFileArray::save_new_game(const char* fileName)
 	{
 		strcpy( last_file_name, gameFile.file_name );
 
-		if( addFlag )
-		{
-			linkin(&gameFile);
+		FilePath full_path(sys.dir_config);
+		full_path += gameFile.file_name;
+		Directory saveGameDirectory;
+		saveGameDirectory.read(full_path, 0);  // 0-Don't sort file names
 
-			quick_sort( sort_game_file_function );
-		}
-		else
+		if( saveGameDirectory.size() == 1 )
 		{
-			game_file_array.update(&gameFile, gameFileRecno);
+			if( addFlag )
+			{
+				linkin(&gameFile);
+
+				quick_sort( sort_game_file_function );
+			}
+			else
+			{
+				this->update(&gameFile, gameFileRecno);
+			}
 		}
 
 		return 1;
 	}
-	return 0;
+	else
+	{
+		box.msg(gameFile.status_str());
+		return 0;
+	}
 }
 //-------- End of function GameFileArray::save_new_game -------//
 
 
+
+//------- Begin of function GameFileArray::set_file_name -------//
+//
+// Set the game file name of the given save game
+//
+// e.g. ENLI_001.SAV - the first saved game of the group "Enlight Enterprise"
+//
+void GameFileArray::set_file_name(char* /*out*/ fileName, int size)
+{
+	enum { NAME_PREFIX_LEN = 4,    // Maximum 4 characters in name prefix, e.g. "ENLI" for "Enlight Enterprise"
+		NAME_NUMBER_LEN = 3  };
+
+	String str, str2;
+	int    i;
+	char   nameChar;
+	const char*  baseName;             // the long name which the file name is based on
+	char   addStr[] = "0";       // as a small string for adding to the large string
+
+	baseName = (~nation_array)->king_name();
+
+	//--------- add the group name prfix ----------//
+
+	for( i=0 ; i<(int) strlen(baseName) && (int) str.len() < NAME_PREFIX_LEN ; i++ )
+	{
+		nameChar = misc.upper(baseName[i]);
+
+		if( ( nameChar >= 'A' && nameChar <= 'Z' ) ||
+			( nameChar >= '0' && nameChar <= '9' ) )
+		{
+			addStr[0] = nameChar;
+
+			str += addStr;
+		}
+	}
+
+	//----- add tailing characters if prefix len < NAME_PREFIX_LEN+1 ---//
+
+	while( str.len() < NAME_PREFIX_LEN+1 )       // +1 is the "_" between the name and the number
+		str += "_";
+
+	//---- find the saved game number for this saved game ----//
+
+	int       curNumber, lastNumber=0;
+
+	for( i=1 ; i<=this->size() ; i++ )
+	{
+		GameFile* findSaveGame = (*this)[i];
+
+		// ##### begin Gilbert 3/10 ########//
+		if( strnicmp(findSaveGame->file_name, str, NAME_PREFIX_LEN)==0 )
+			// ##### end Gilbert 3/10 ########//
+		{
+			//------------------------------------------------//
+			//
+			// if there is a free number in the middle of the list
+			// (left by being deleted game), use this number.
+			//
+			//------------------------------------------------//
+
+			curNumber = atoi( findSaveGame->file_name+NAME_PREFIX_LEN+1 );      // +1 is to pass the "_" between the name and the number
+
+			if( curNumber > lastNumber+1 )   // normally, curNumber should be lastNumber+1
+				break;
+
+			lastNumber = curNumber;
+		}
+	}
+
+	//------- add saved game number after the prefix --------//
+
+	str2 = lastNumber+1;    // use the next number after the last number
+
+	for( i=NAME_NUMBER_LEN-str2.len() ; i>0 ; i-- )   // add "0" before the number if the len of the number < NAME_NUMBER_LEN
+		str += "0";
+
+	str += str2;
+	str += ".SAV";
+
+	//----- copy the string to fileName ------//
+
+	strncpy( fileName, str, size-1 );
+	fileName[size-1] = '\0';
+}
+//--------- End of function GameFileArray::set_file_name -------//
+
+
 //------- Begin of function GameFileArray::del_game ------//
 
 void GameFileArray::del_game()
 {
-	char full_path[MAX_PATH+1];
 	int recNo = browse_recno;
 
-	if( !box.ask( _("This saved game will be deleted, proceed ?") ) )
+	if( !box.ask( _("This saved game will be deleted. Proceed?") ) )
 		return;
 
-	//---------------------------------------------------//
-
-	if (!misc.path_cat(full_path, sys.dir_config, game_file_array[recNo]->file_name, MAX_PATH))
-	{
-		ERR("Path to the saved game too long.\n");
+	FilePath fullPath(sys.dir_config);
+	fullPath += (*this)[recNo]->file_name;
+	if( fullPath.error_flag )
 		return;
-	}
-
-	unlink(full_path);   // unlink() is a C standard function in io.h, for delete file
+	unlink(fullPath);
 
 	go(recNo);
 	linkout();
@@ -823,120 +959,40 @@ void GameFileArray::del_game()
 //--------- End of function GameFileArray::del_game ------//
 
 
-//------- Begin of function GameFileArray::write_hall_of_fame ------//
-//
-int GameFileArray::write_hall_of_fame()
-{
-	char full_path[MAX_PATH+1];
-	int  rc;
-	File file;
-
-	if (!misc.path_cat(full_path, sys.dir_config, HALL_OF_FAME_FILE_NAME, MAX_PATH))
-	{
-		ERR("Path to the hall of fame too long.\n");
-		return 0;
-	}
-
-	rc = file.file_create( full_path, 0, 1 );  // 0=don't handle error itself
-
-	if( !rc )
-		return 0;
-																			// 1=allow the writing size and the read size to be different
-	//--------- Write Hall of Fame ----------//
-
-	if( rc )
-		rc = file.file_write( hall_fame_array, sizeof(HallFame) * HALL_FAME_NUM );
-
-	//------ write last saved game file name ------//
-
-	if( rc )
-		rc = file.file_write( last_file_name, MAX_PATH+1 );
-
-	file.file_close();
-	
-	return rc;
-}
-//--------- End of function GameFileArray::write_hall_of_fame ------//
-
-
-//------- Begin of function GameFileArray::read_hall_of_fame ------//
-//
-int GameFileArray::read_hall_of_fame()
-{
-	char full_path[MAX_PATH+1];
-	int  rc;
-	File file;
-
-	if (!misc.path_cat(full_path, sys.dir_config, HALL_OF_FAME_FILE_NAME, MAX_PATH))
-	{
-		ERR("Path to the hall of fame too long.\n");
-		return 0;
-	}
-
-	if( !misc.is_file_exist(full_path) )
-		return 0;
-
-	rc = file.file_open(full_path, 0, 1);   // 0=don't handle error itself
-                                                        // 1=allow the writing size and the read size to be different
-	if( !rc )
-		return 0;
-																			// 1=allow the writing size and the read size to be different
-	//--------- Read Hall of Fame ----------//
-
-	if( rc )
-		rc = file.file_read( hall_fame_array, sizeof(HallFame) * HALL_FAME_NUM );
-
-	//------ read last saved game file name ------//
-
-	if( rc )
-		rc = file.file_read( last_file_name, MAX_PATH+1);
-
-	file.file_close();
-
-	return rc;
-}
-//--------- End of function GameFileArray::read_hall_of_fame ------//
-
-
 //-------- Begin of function GameFileArray::load_all_game_header --------//
 //
 // Load all headers of all saved game files in current directory.
 //
 void GameFileArray::load_all_game_header(const char *extStr)
 {
-	char full_path[MAX_PATH+1];
-	int       i;
-	Directory gameDir;
-	GameFile  gameFile;
-	File      file;
-
-	if (!misc.path_cat(full_path, sys.dir_config, extStr, MAX_PATH))
-	{
-		ERR("Path to the config directory too long.\n");
-		return;
-	}
-	gameDir.read(full_path, 0);  // 0-Don't sort file names
-
-	//-------- read in the headers of all game sets -------//
-
 	zap();
 
-	for( i=1 ; i<=gameDir.size() ; i++ )
+	FilePath full_path(sys.dir_config);
+
+	full_path += extStr;
+	if( full_path.error_flag )
+		return;
+
+	Directory gameDir;
+	gameDir.read(full_path, 0);  // 0-Don't sort file names
+
+	//-------- read in the headers of all savegames -------//
+
+	for( int i=1 ; i<=gameDir.size() ; i++ )
 	{
-		if (!misc.path_cat(full_path, sys.dir_config, gameDir[i]->name, MAX_PATH))
-		{
-			ERR("Path to the saved game too long.\n");
+		const char* const gameFileName = gameDir[i]->name;
+		FilePath save_game_path(sys.dir_config);
+
+		save_game_path += gameFileName;
+		if( save_game_path.error_flag )
 			continue;
-		}
-		if( file.file_open(full_path, 1, 1)      // last 1=allow varying read & write size
-			&& file.file_read(&gameFile, sizeof(GameFile))
-			&& gameFile.validate_header() )
+
+		GameFile gameFile;
+		if( gameFile.read_game_header(save_game_path) )
 		{
-			strcpy( gameFile.file_name, gameDir[i]->name );  // in case that the name may be different
-			gameFile.file_date = gameDir[i]->time;
-	      linkin(&gameFile);
+			strcpy( gameFile.file_name, gameFileName );  // in case that the name may be different
+			linkin(&gameFile);
 		}
-		file.file_close();
 	}
 
 	quick_sort( sort_game_file_function );
@@ -968,23 +1024,6 @@ static int sort_game_file_function( const void *a, const void *b )
 //------- End of function sort_game_file_function ------//
 
 
-//------ Begin of function last_game_recno ------//
-//
-static int last_game_recno()
-{
-	int i;
-
-	for( i=game_file_array.size() ; i>0 ; i-- )
-	{
-		if( strcmp(game_file_array[i]->file_name, game_file_array.last_file_name)==0 )
-			return i;
-	}
-
-	return 0;    // if none of them match, return 1 as the first record
-}
-//------- End of function last_game_recno ------//
-
-
 //------- Begin of function GameFileArray::operator[] -----//
 
 GameFile* GameFileArray::operator[](int recNo)
@@ -996,37 +1035,15 @@ GameFile* GameFileArray::operator[](int recNo)
 //--------- End of function GameFileArray::operator[] ----//
 
 
-//------- Begin of static function key_search --------//
-
-static void key_search()
-{
-	int i;
-	int searchKey = misc.upper(mouse.key_code);
-
-	if( searchKey < '0' || searchKey > 'Z' )
-		return;
-
-	for( i=1 ; i<=game_file_array.size() ; i++ )
-	{
-		if( misc.upper(game_file_array[i]->file_name[0]) >= searchKey )
-		{
-			browse_recno = i+(action_mode==1);
-			return;
-		}
-	}
-}
-//------- End of static function key_search --------//
-
-
 //------- Begin of static function disp_scroll_bar_func --------//
 static void disp_scroll_bar_func(SlideVBar *scroll, int)
 {
-	short rectTop = scroll->rect_top();
-	short rectBottom = scroll->rect_bottom();
-	vga_front.bar( scroll->scrn_x1, rectTop, scroll->scrn_x2, rectBottom, VGA_YELLOW+1);
-	if( rectBottom - rectTop > 6 )
-	{
-		vga_front.d3_panel_up(scroll->scrn_x1, rectTop, scroll->scrn_x2, rectBottom,2,0);
-	}
+    short rectTop = scroll->rect_top();
+    short rectBottom = scroll->rect_bottom();
+    vga_front.bar( scroll->scrn_x1, rectTop, scroll->scrn_x2, rectBottom, VGA_YELLOW+1);
+    if( rectBottom - rectTop > 6 )
+    {
+        vga_front.d3_panel_up(scroll->scrn_x1, rectTop, scroll->scrn_x2, rectBottom,2,0);
+    }
 }
 //------- End of static function disp_scroll_bar_func --------//
