@@ -55,6 +55,10 @@ enum { TALK_LINE_HEIGHT = 18 };
 
 static String nation_name_str_array[MAX_NATION];
 
+// ----------- Define static function ----------//
+
+static char* select_nation_color(char nation_color);
+
 //---------- Begin of function TalkRes::TalkRes -----------//
 //
 TalkRes::TalkRes() : talk_msg_array( sizeof(TalkMsg), 100 )
@@ -272,11 +276,7 @@ int TalkRes::add_trade_embargo_choices()
 		{
 			//------ add color bar -------//
 
-			char colorCodeStr[] = "0 ";
-
-			colorCodeStr[0] = FIRST_NATION_COLOR_CODE_IN_TEXT + nation_array[i]->color_scheme_id;
-
-			nation_name_str_array[i-1] = colorCodeStr;
+			nation_name_str_array[i-1] = select_nation_color(nation_array[i]->color_scheme_id);
 
 			//------ add natino name ------//
 
@@ -317,11 +317,7 @@ int TalkRes::add_declare_war_choices()
 		{
 			//------ add color bar -------//
 
-			char colorCodeStr[] = "0 ";
-
-			colorCodeStr[0] = FIRST_NATION_COLOR_CODE_IN_TEXT + nation_array[i]->color_scheme_id;
-
-			nation_name_str_array[i-1] = colorCodeStr;
+			nation_name_str_array[i-1] = select_nation_color(nation_array[i]->color_scheme_id);
 
 			//------ add natino name ------//
 
@@ -720,6 +716,13 @@ int TalkRes::ai_send_talk_msg(int toNationRecno, int fromNationRecno, int talkId
 // Now records in talk_msg_array cannot be deleted as
 // news_array.diplomacy() use recno to refer to talk_msg_array.
 //
+//###### begin jesse 2022/10/2 #######//
+// This function calls linkin (send_talk_msg_now). This must not be called from
+// the TalkMsg class. The talkMsgPtr argument must not be in talk_msg_array.
+// If the calling function is referencing any pointers into talk_msg_array,
+// they must be treated as invalid after calling this function.
+//###### end jesse 2022/10/2 #######//
+//
 void TalkRes::send_talk_msg(TalkMsg* talkMsgPtr, char remoteAction)
 {
 	//-------- send multiplayer -----------//
@@ -763,7 +766,7 @@ void TalkRes::send_talk_msg(TalkMsg* talkMsgPtr, char remoteAction)
 	//---- if it's a notification message ----//
 
 	if( talkMsgPtr->reply_type == REPLY_NOT_NEEDED )
-		talkMsgPtr->process_accepted_reply();
+		process_accepted_reply(talkMsgPtr);
 }
 //-------- End of function TalkRes::send_talk_msg ---------//
 
@@ -792,7 +795,8 @@ void TalkRes::send_talk_msg_now(TalkMsg* talkMsgPtr)
 			news_array.diplomacy( talk_msg_array.recno() );
 			// ###### begin Gilbert 9/10 ########//
 			// sound effect
-			se_ctrl.immediate_sound(talkMsgPtr->talk_id==TALK_DECLARE_WAR ? (char*)"DECL_WAR":(char*)"GONG");
+			if( toNation->get_relation(talkMsgPtr->from_nation_recno)->has_contact )
+				se_ctrl.immediate_sound(talkMsgPtr->talk_id==TALK_DECLARE_WAR ? (char*)"DECL_WAR":(char*)"GONG");
 			// ###### end Gilbert 9/10 ########//
 			break;
 
@@ -868,6 +872,9 @@ void TalkRes::reply_talk_msg(int talkMsgRecno, char replyType, char remoteAction
 
 		case NATION_AI:
 			fromNation->ai_notify_reply( talkMsgRecno );		// notify the AI nation about this reply.
+			if( is_talk_msg_deleted(talkMsgRecno) )
+				return;
+			talkMsgPtr = get_talk_msg(talkMsgRecno);
 			break;
 
 		case NATION_REMOTE:
@@ -877,12 +884,18 @@ void TalkRes::reply_talk_msg(int talkMsgRecno, char replyType, char remoteAction
 	//------- if the offer is accepted -------//
 
 	if( talkMsgPtr->reply_type == REPLY_ACCEPT )
-		talkMsgPtr->process_accepted_reply();
+	{
+		process_accepted_reply(talkMsgPtr);
+		if( is_talk_msg_deleted(talkMsgRecno) )
+			return;
+		talkMsgPtr = get_talk_msg(talkMsgRecno);
+	}
 
 	//--- if the player has replyed the message, remove it from the news display ---//
 
 	if( talkMsgPtr->to_nation_recno == nation_array.player_recno )
 		news_array.remove(NEWS_DIPLOMACY, talkMsgRecno);
+
 }
 //-------- End of function TalkRes::reply_talk_msg ---------//
 
@@ -1087,7 +1100,7 @@ int TalkRes::detect_talk()
 		choice_question_second_line = NULL;
 
 		talk_choice_count = 0;
-		add_talk_choice( _("Continue"), 0 );
+		add_talk_choice( _("Continue."), 0 );
 	}
 
 	return 1;
@@ -1162,6 +1175,7 @@ void TalkRes::process_talk_msg()
 			talkMsg->reply_type = REPLY_REJECT;
 
 			nation_array[talkMsg->from_nation_recno]->ai_notify_reply(i);
+			talkMsg = get_talk_msg(i); // in case talkMsg ptr was invalidated in resize
 		}
 
 		//--- delete the talk message after a year ---//
@@ -1376,3 +1390,16 @@ void TalkRes::del_all_nation_msg(int nationRecno)
 }
 //-------- End of function TalkRes::del_all_nation_msg ---------//
 
+
+#define ASCII_ZERO 0x30
+//------ Begin of static function select_nation_color ------//
+//
+static char* select_nation_color(char nation_color)
+{
+	static char colorCodeStr[] = "@COL0 ";
+
+	colorCodeStr[4] = ASCII_ZERO + nation_color;
+
+	return colorCodeStr;
+}
+//------ End of static function select_nation_color ------//
