@@ -95,18 +95,18 @@ int Unit::update_attack_path_dist()
 //	The function can be called in by class Bullet since parent and target
 //	are specified.
 //
-// [Unit*]	parentUnit is the unit attacking
-//	[Unit*]	taregtUnit is the unit being attacked
-// <int>		attackDamage is the damage done to the target Unit
+// <Unit*>  parentUnit is the unit attacking
+// <Unit*>  targetUnit is the unit being attacked
+// <int>    attackDamage is the damage done to the target Unit
+// <short>  parentNationRecno - the nation that ordered the hit
 //
 // ****************************** Warning ***********************************
 // don't use any member variables of this unit. This unit may not be involved
 // in the attack event
 // **************************************************************************
 //
-void Unit::hit_target(Unit* parentUnit, Unit* targetUnit, float attackDamage)
+void Unit::hit_target(Unit* parentUnit, Unit* targetUnit, float attackDamage, short parentNationRecno)
 {
-	short	parentNationRecno = (parentUnit) ? parentUnit->nation_recno : 0;
 	short	targetNationRecno = targetUnit->nation_recno;
 
 	//------------------------------------------------------------//
@@ -178,13 +178,13 @@ void Unit::hit_target(Unit* parentUnit, Unit* targetUnit, float attackDamage)
 			{
 				if( targetNationRecno )
 				{
-					targetNationPtr->civilian_killed(targetUnit->race_id, 0);
+					targetNationPtr->civilian_killed(targetUnit->race_id, 0, 1);
 					targetNationPtr->own_civilian_killed++;
 				}
 
-				if(parentUnit && parentNationRecno )
+				if( parentNationPtr )
 				{
-					parentNationPtr->civilian_killed(targetUnit->race_id, 1);
+					parentNationPtr->civilian_killed(targetUnit->race_id, 1, 1);
 					parentNationPtr->enemy_civilian_killed++;
 				}
 			}
@@ -192,13 +192,13 @@ void Unit::hit_target(Unit* parentUnit, Unit* targetUnit, float attackDamage)
 			{
 				if( targetNationRecno )
 				{
-					targetNationPtr->civilian_killed(targetUnit->race_id, 0);
+					targetNationPtr->civilian_killed(targetUnit->race_id, 0, 0);
 					targetNationPtr->own_civilian_killed++;
 				}
 
-				if(parentUnit && parentNationRecno )
+				if( parentNationPtr )
 				{
-					parentNationPtr->civilian_killed(targetUnit->race_id, 0);
+					parentNationPtr->civilian_killed(targetUnit->race_id, 1, 0);
 					parentNationPtr->enemy_civilian_killed++;
 				}
 			}
@@ -207,7 +207,7 @@ void Unit::hit_target(Unit* parentUnit, Unit* targetUnit, float attackDamage)
 				if( targetNationRecno )
 					targetNationPtr->own_soldier_killed++;
 
-				if(parentUnit && parentNationRecno )
+				if( parentNationPtr )
 					parentNationPtr->enemy_soldier_killed++;
 			}
 		}
@@ -219,7 +219,7 @@ void Unit::hit_target(Unit* parentUnit, Unit* targetUnit, float attackDamage)
 			switch( unit_res[targetUnit->unit_id]->unit_class )
 			{
 				case UNIT_CLASS_WEAPON:
-					if(parentUnit && parentNationRecno )
+					if( parentNationPtr )
 						parentNationPtr->enemy_weapon_destroyed++;
 
 					if( targetNationRecno )
@@ -228,7 +228,7 @@ void Unit::hit_target(Unit* parentUnit, Unit* targetUnit, float attackDamage)
 
 
 				case UNIT_CLASS_SHIP:
-					if(parentUnit && parentNationRecno )
+					if( parentNationPtr )
 						parentNationPtr->enemy_ship_destroyed++;
 
 					if( targetNationRecno )
@@ -243,10 +243,10 @@ void Unit::hit_target(Unit* parentUnit, Unit* targetUnit, float attackDamage)
 			{
 				// Race-Id of 0 means a loyalty penalty applied for all races
 				if( targetNationRecno )
-					targetNationPtr->civilian_killed(0, -1);
+					targetNationPtr->civilian_killed(0, 0, 3);
 
-				if( parentUnit && parentNationRecno )
-					parentNationPtr->civilian_killed(0, 3);
+				if( parentNationPtr )
+					parentNationPtr->civilian_killed(0, 1, 3);
 			}
 		}
 
@@ -273,7 +273,7 @@ void Unit::hit_target(Unit* parentUnit, Unit* targetUnit, float attackDamage)
 
 	//------- two nations at war ---------//
 
-	if( parentNationRecno && targetNationRecno )
+	if( parentNationPtr && targetNationRecno )
 	{
 		parentNationPtr->set_at_war_today();
 		targetNationPtr->set_at_war_today(parentUnit->sprite_recno);
@@ -281,7 +281,7 @@ void Unit::hit_target(Unit* parentUnit, Unit* targetUnit, float attackDamage)
 
 	//-------- increase battling fryhtan score --------//
 
-	if( targetUnitClass==UNIT_CLASS_MONSTER && parentNationRecno )
+	if( parentNationPtr && targetUnitClass==UNIT_CLASS_MONSTER )
 	{
 		parentNationPtr->kill_monster_score += (float) 0.1;
 	}
@@ -320,8 +320,22 @@ void Unit::hit_target(Unit* parentUnit, Unit* targetUnit, float attackDamage)
 
 	else if( targetUnitClass == UNIT_CLASS_MONSTER )
 	{
-		if( parentNationRecno )
+		if( parentNationPtr )
 			parentNationPtr->change_reputation(REPUTATION_INCREASE_PER_ATTACK_MONSTER);
+
+		//--- if a member in a troop is under attack, ask for other troop members to help ---//
+
+		if( info.game_date%2 == sprite_recno%2 )
+		{
+			if( targetUnit->leader_unit_recno ||
+				 (targetUnit->team_info && targetUnit->team_info->member_count > 1) )
+			{
+				if( !unit_array.is_deleted(parentUnit->sprite_recno) )		// it is possible that parentUnit is dying right now 
+				{
+					targetUnit->ask_team_help_attack(parentUnit);
+				}
+			}
+		}
 	}
 
 	//------------------------------------------//
@@ -380,8 +394,7 @@ void Unit::unit_auto_guarding(Unit *attackUnit)
 			changeToAttack++;  //else continue to attack the target unit
 		else
 		{
-			err_when(!action_para);
-			if(unit_array.is_deleted(action_para))
+			if(!action_para || unit_array.is_deleted(action_para))
 				changeToAttack++; // attack new target
 		}
 	}
@@ -509,23 +522,24 @@ void Unit::unit_auto_guarding(Unit *attackUnit)
 //			In range attack, the unit calling this function may not be the
 //			attacking unit.
 //
-// <Unit*> parentUnit   - the attacking unit
-// <int>	  target?Loc   - the target building location
+// <Unit*> attackUnit   - the attacking unit
+// <int>   target?Loc   - the target building location
 // <int>   attackDamage - the actual damage made
+// <short> attackNationRecno - the nation that ordered the hit
 //
 // ****************************** Warning ***********************************
 // don't use any member variables of this unit. This unit may not be involved
 // in the attack event
 // **************************************************************************
 //
-void Unit::hit_building(Unit* attackUnit, int targetXLoc, int targetYLoc, float attackDamage)
+void Unit::hit_building(Unit* attackUnit, int targetXLoc, int targetYLoc, float attackDamage, short attackNationRecno)
 {
 	Location* locPtr = world.get_loc(targetXLoc, targetYLoc);
 
 	if(locPtr->is_firm())
-		hit_firm( attackUnit, targetXLoc, targetYLoc, attackDamage);
+		hit_firm(attackUnit, targetXLoc, targetYLoc, attackDamage, attackNationRecno);
 	else if( locPtr->is_town() )
-		hit_town( attackUnit, targetXLoc, targetYLoc, attackDamage);
+		hit_town(attackUnit, targetXLoc, targetYLoc, attackDamage, attackNationRecno);
 }
 //---------- End of function Unit::hit_building ----------//
 
@@ -539,15 +553,16 @@ void Unit::hit_building(Unit* attackUnit, int targetXLoc, int targetYLoc, float
 //			attacking unit.
 //
 // <Unit*> attackUnit   - the attacking unit
-// <int>	  target?Loc   - the target building location
+// <int>   target?Loc   - the target building location
 // <int>   attackDamage - the actual damage made
+// <short> attackNationRecno - the nation that ordered the hit
 //
 // ****************************** Warning ***********************************
 // don't use any member variables of this unit. This unit may not be involved
 // in the attack event
 // **************************************************************************
 //
-void Unit::hit_firm(Unit* attackUnit, int targetXLoc, int targetYLoc, float attackDamage)
+void Unit::hit_firm(Unit* attackUnit, int targetXLoc, int targetYLoc, float attackDamage, short attackNationRecno)
 {
 	Location* locPtr = world.get_loc(targetXLoc, targetYLoc);
 	if(!locPtr->is_firm())
@@ -558,38 +573,40 @@ void Unit::hit_firm(Unit* attackUnit, int targetXLoc, int targetYLoc, float atta
 	Firm *targetFirm = firm_array[locPtr->firm_recno()];
 	err_when(!targetFirm);
 
+	Nation *attackNationPtr = nation_array.is_deleted(attackNationRecno) ? NULL : nation_array[attackNationRecno];
+
 	//------------------------------------------------------------------------------//
 	// change relation to hostile
 	// check for NULL to skip unhandled case by bullets
 	// check for SPRITE_DIE to skip the case by EXPLOSIVE_CART
 	//------------------------------------------------------------------------------//
 	if( attackUnit!=NULL && attackUnit->cur_action!=SPRITE_DIE &&
-		 targetFirm->nation_recno != attackUnit->nation_recno )		// the target and the attacker's nations are different (it's possible that when a unit who has just changed nation has its bullet hitting its own nation)
+		 targetFirm->nation_recno != attackNationRecno )		// the target and the attacker's nations are different (it's possible that when a unit who has just changed nation has its bullet hitting its own nation)
 	{
-		if( attackUnit->nation_recno && targetFirm->nation_recno )
+		if( attackNationPtr && targetFirm->nation_recno )
 		{
 			//### trevor 29/9 ###//
-			nation_array[attackUnit->nation_recno]->set_at_war_today();
+			attackNationPtr->set_at_war_today();
 			nation_array[targetFirm->nation_recno]->set_at_war_today(attackUnit->sprite_recno);
 			//### trevor 29/9 ###//
 		}
 
 		if( targetFirm->nation_recno )
-			nation_array[targetFirm->nation_recno]->being_attacked(attackUnit->nation_recno);
+			nation_array[targetFirm->nation_recno]->being_attacked(attackNationRecno);
 
 		//------------ auto defense -----------------//
 		if(attackUnit->is_visible())
 			targetFirm->auto_defense(attackUnit->sprite_recno);
 
-		if(attackUnit->nation_recno!=targetFirm->nation_recno)
+		if( attackNationRecno != targetFirm->nation_recno )
 			attackUnit->gain_experience(); // gain experience to increase combat level
 
 		targetFirm->being_attacked(attackUnit->sprite_recno);
 
 		//------ increase battling fryhtan score -------//
 
-		if( targetFirm->firm_id == FIRM_MONSTER && attackUnit->nation_recno )
-			nation_array[attackUnit->nation_recno]->kill_monster_score += (float) 0.01;
+		if( attackNationPtr && targetFirm->firm_id == FIRM_MONSTER )
+			attackNationPtr->kill_monster_score += (float) 0.01;
 	}
 
 	//---------- add indicator on the map ----------//
@@ -599,6 +616,9 @@ void Unit::hit_firm(Unit* attackUnit, int targetXLoc, int targetYLoc, float atta
 		war_point_array.add_point(targetFirm->center_x, targetFirm->center_y);
 	// ###### end Gilbert 6/10 #######//
 
+	// update attack date in case being_attacked() is not called
+	targetFirm->last_attacked_date = info.game_date;
+
 	//---------- damage to the firm ------------//
 
 	targetFirm->hit_points -= attackDamage/3;		// /3 so that it takes longer to destroy a firm
@@ -613,15 +633,14 @@ void Unit::hit_firm(Unit* attackUnit, int targetXLoc, int targetYLoc, float atta
 			'F', targetFirm->firm_id, "DIE" );
 
 		if( targetFirm->nation_recno == nation_array.player_recno )
-			news_array.firm_destroyed(targetFirm->firm_recno, attackUnit);
+			news_array.firm_destroyed(targetFirm->firm_recno, attackUnit, attackNationRecno);
 
 		if( targetFirm->nation_recno )
 		{
-			if( attackUnit->nation_recno )
-				nation_array[attackUnit->nation_recno]->enemy_firm_destroyed++;
+			if( attackNationPtr )
+				attackNationPtr->enemy_firm_destroyed++;
 
-			if( targetFirm->nation_recno )
-				nation_array[targetFirm->nation_recno]->own_firm_destroyed++;
+			nation_array[targetFirm->nation_recno]->own_firm_destroyed++;
 		}
 
 		else if( targetFirm->firm_id == FIRM_MONSTER )
@@ -644,15 +663,16 @@ void Unit::hit_firm(Unit* attackUnit, int targetXLoc, int targetYLoc, float atta
 //			attacking unit.
 //
 // <Unit*> attackUnit   - the attacking unit
-// <int>	  target?Loc   - the target building location
+// <int>   target?Loc   - the target building location
 // <int>   attackDamage - the actual damage made
+// <short> attackNationRecno - the nation that ordered the hit
 //
 // ****************************** Warning ***********************************
 // don't use any member variables of this unit. This unit may not be involved
 // in the attack event
 // **************************************************************************
 //
-void Unit::hit_town(Unit* attackUnit, int targetXLoc, int targetYLoc, float attackDamage)
+void Unit::hit_town(Unit* attackUnit, int targetXLoc, int targetYLoc, float attackDamage, short attackNationRecno)
 {
 	Location *locPtr = world.get_loc(targetXLoc, targetYLoc);
 
@@ -681,23 +701,23 @@ void Unit::hit_town(Unit* attackUnit, int targetXLoc, int targetYLoc, float atta
 	// check for SPRITE_DIE to skip the case by EXPLOSIVE_CART
 	//------------------------------------------------------------------------------//
 	if( attackUnit!=NULL && attackUnit->cur_action!=SPRITE_DIE &&
-		 targetTown->nation_recno != attackUnit->nation_recno )		// the target and the attacker's nations are different (it's possible that when a unit who has just changed nation has its bullet hitting its own nation)
+		 targetTown->nation_recno != attackNationRecno )		// the target and the attacker's nations are different (it's possible that when a unit who has just changed nation has its bullet hitting its own nation)
 	{
 		int townNationRecno = targetTown->nation_recno;
 
 		//------- change to hostile relation -------//
 
-		if( attackUnit->nation_recno && targetTown->nation_recno )
+		if( attackNationRecno && targetTown->nation_recno )
 		{
 			//### trevor 29/9 ###//
-			nation_array[attackUnit->nation_recno]->set_at_war_today();
+			nation_array[attackNationRecno]->set_at_war_today();
 			nation_array[targetTown->nation_recno]->set_at_war_today(attackUnit->sprite_recno);
 			//### trevor 29/9 ###//
 		}
 
 		if( targetTown->nation_recno)
 		{
-			nation_array[targetTown->nation_recno]->being_attacked(attackUnit->nation_recno);
+			nation_array[targetTown->nation_recno]->being_attacked(attackNationRecno);
 		}
 
 		news_array.disable();		// don't add the town abandon news that might be called by Town::dec_pop() as the town is actually destroyed not abandoned
@@ -711,12 +731,12 @@ void Unit::hit_town(Unit* attackUnit, int targetXLoc, int targetYLoc, float atta
 		if( town_array.is_deleted(targetTownRecno) &&
 			 townNationRecno == nation_array.player_recno )
 		{
-			news_array.town_destroyed(targetTownNameId, targetTownXLoc, targetTownYLoc, attackUnit);
+			news_array.town_destroyed(targetTownNameId, targetTownXLoc, targetTownYLoc, attackUnit, attackNationRecno);
 		}
 
 		//---------- gain experience --------//
 
-		if(attackUnit->nation_recno!=targetTown->nation_recno)
+		if( attackNationRecno != targetTown->nation_recno )
 			attackUnit->gain_experience(); // gain experience to increase combat level
 
 		//------------ auto defense -----------------//
@@ -733,15 +753,16 @@ void Unit::hit_town(Unit* attackUnit, int targetXLoc, int targetYLoc, float atta
 //--------- Begin of function Unit::hit_wall -----------//
 //
 // <Unit*> attackUnit   - the attacking unit
-// <int>	  target?Loc   - the targeted wall location
+// <int>   target?Loc   - the targeted wall location
 // <int>   attackDamage - the actual damage made
+// <short> attackNationRecno - the nation that ordered the hit
 //
 // ****************************** Warning ***********************************
 // don't use any member variables of this unit. This unit may not be involved
 // in the attack event
 // **************************************************************************
 //
-void Unit::hit_wall(Unit* attackUnit, int targetXLoc, int targetYLoc, float attackDamage)
+void Unit::hit_wall(Unit* attackUnit, int targetXLoc, int targetYLoc, float attackDamage, short attackNationRecno)
 {
 	Location *locPtr = world.get_loc(targetXLoc, targetYLoc);
 	err_when(!locPtr->is_wall());
@@ -749,7 +770,7 @@ void Unit::hit_wall(Unit* attackUnit, int targetXLoc, int targetYLoc, float atta
 	//######## begin trevor 25/6 #########//
 /*
 	if(attackUnit!=NULL)
-		attackUnit->change_relation(attackUnit->nation_recno, locPtr->wall_nation_recno(), NATION_HOSTILE);
+		attackUnit->change_relation(attackNationRecno, locPtr->wall_nation_recno(), NATION_HOSTILE);
 */
 	//######## end trevor 25/6 #########//
 
@@ -817,38 +838,10 @@ float Unit::actual_damage()
 	//
 	//------------------------------------------------------------------------//
 
-	if( leader_unit_recno )
+	if( is_leader_in_range() )
 	{
-		if( unit_array.is_deleted(leader_unit_recno) )
-		{
-			leader_unit_recno = 0;
-		}
-		else
-		{
-			Unit* leaderUnit = unit_array[leader_unit_recno];
-			int	leaderXLoc, leaderYLoc;
-
-			if( leaderUnit->is_visible() )
-			{
-				leaderXLoc = cur_x_loc();
-				leaderYLoc = cur_y_loc();
-			}
-			else if( leaderUnit->unit_mode == UNIT_MODE_OVERSEE )
-			{
-				Firm* firmPtr = firm_array[leaderUnit->unit_mode_para];
-
-				leaderXLoc = firmPtr->center_x;
-				leaderYLoc = firmPtr->center_y;
-			}
-			else
-				leaderXLoc = -1;
-
-			if( leaderXLoc >= 0 &&
-				 misc.points_distance(cur_x_loc(), cur_y_loc(), leaderXLoc, leaderYLoc) <= EFFECTIVE_LEADING_DISTANCE )
-			{
-				attackDamage += attackDamage * leaderUnit->skill.skill_level / 100;
-			}
-		}
+		Unit *leaderUnit = unit_array[leader_unit_recno];
+		attackDamage += attackDamage * leaderUnit->skill.skill_level / 100;
 	}
 
 	return (float) attackDamage / ATTACK_SLOW_DOWN;		// lessen all attacking damages, thus slowing down all battles.
@@ -879,38 +872,18 @@ void Unit::gain_experience()
 
 	//--- if this is a soldier led by a commander, increase the leadership of its commander -----//
 
-	if( leader_unit_recno )
+	if( is_leader_in_range() )
 	{
 		Unit* leaderUnit = unit_array[leader_unit_recno];
-		int	leaderXLoc= -1, leaderYLoc;
 
-		if( leaderUnit->is_visible() )
+		leaderUnit->inc_minor_skill_level(1);
+
+		//-- give additional increase if the leader has skill potential on leadership --//
+
+		if( leaderUnit->skill.skill_potential > 0 )
 		{
-			leaderXLoc = cur_x_loc();
-			leaderYLoc = cur_y_loc();
-		}
-		else if( leaderUnit->unit_mode == UNIT_MODE_OVERSEE )
-		{
-			Firm* firmPtr = firm_array[leaderUnit->unit_mode_para];
-
-			leaderXLoc = firmPtr->center_x;
-			leaderYLoc = firmPtr->center_y;
-		}
-		else
-			leaderXLoc = -1;
-
-		if( leaderXLoc >= 0 &&
-			 misc.points_distance( cur_x_loc(), cur_y_loc(), leaderXLoc, leaderYLoc ) <= EFFECTIVE_LEADING_DISTANCE )
-		{
-			leaderUnit->inc_minor_skill_level(1);
-
-			//-- give additional increase if the leader has skill potential on leadership --//
-
-			if( leaderUnit->skill.skill_potential > 0 )
-			{
-				if( misc.random(10-leaderUnit->skill.skill_potential/10)==0 )
-					leaderUnit->inc_minor_skill_level(5);
-			}
+			if( misc.random(10-leaderUnit->skill.skill_potential/10)==0 )
+				leaderUnit->inc_minor_skill_level(5);
 		}
 
 		//--- if this soldier has leadership potential and is led by a commander ---//
@@ -1054,10 +1027,10 @@ void Unit::choose_best_attack_mode(int attackDistance, char targetMobileType)
 	//return;
 
 	//-------------------- define parameters -----------------------//
-	UCHAR attackModeBeingUsed = cur_attack;
+	uint8_t attackModeBeingUsed = cur_attack;
 	err_when(attackModeBeingUsed<0 || attackModeBeingUsed>MAX_UNIT_ATTACK_TYPE);
 	//UCHAR maxAttackRangeMode = 0;
-	UCHAR maxAttackRangeMode = cur_attack;
+	uint8_t maxAttackRangeMode = cur_attack;
 	AttackInfo* attackInfoMaxRange = attack_info_array;
 	AttackInfo* attackInfoChecking;
 	AttackInfo* attackInfoSelected = attack_info_array+cur_attack;
@@ -1072,7 +1045,7 @@ void Unit::choose_best_attack_mode(int attackDistance, char targetMobileType)
 		int canAttack = 0;
 		int checkingDamageWeight, selectedDamageWeight;
 
-		for(UCHAR i=0; i<attack_count; i++)
+		for(uint8_t i=0; i<attack_count; i++)
 		{
 			if(attackModeBeingUsed==i)
 				continue; // it is the mode already used
@@ -1085,7 +1058,7 @@ void Unit::choose_best_attack_mode(int attackDistance, char targetMobileType)
 
 				if(attackInfoSelected->attack_range<attackDistance)
 				{
-					attackModeBeingUsed = UCHAR(i);
+					attackModeBeingUsed = i;
 					attackInfoSelected = attackInfoChecking;
 					continue;
 				}
@@ -1100,14 +1073,14 @@ void Unit::choose_best_attack_mode(int attackDistance, char targetMobileType)
 					{
 						if(attackInfoChecking->attack_range==attackDistance && checkingDamageWeight>selectedDamageWeight)
 						{
-							attackModeBeingUsed = UCHAR(i); // choose the one with strongest damage
+							attackModeBeingUsed = i; // choose the one with strongest damage
 							attackInfoSelected = attackInfoChecking;
 						}
 						continue;
 					}
 					else if(attackInfoChecking->attack_range==1)
 					{
-						attackModeBeingUsed = UCHAR(i);
+						attackModeBeingUsed = i;
 						attackInfoSelected = attackInfoChecking;
 						continue;
 					}
@@ -1125,7 +1098,7 @@ void Unit::choose_best_attack_mode(int attackDistance, char targetMobileType)
 							//--------------------------------------------------------------------------//
 							// select one with shortest attack_range
 							//--------------------------------------------------------------------------//
-							attackModeBeingUsed = UCHAR(i);
+							attackModeBeingUsed = i;
 							attackInfoSelected = attackInfoChecking;
 						}
 					}
@@ -1135,7 +1108,7 @@ void Unit::choose_best_attack_mode(int attackDistance, char targetMobileType)
 					//--------------------------------------------------------------------------//
 					// select one that can do the attacking immediately with the strongest damage point
 					//--------------------------------------------------------------------------//
-					attackModeBeingUsed = UCHAR(i);
+					attackModeBeingUsed = i;
 					attackInfoSelected = attackInfoChecking;
 				}
 			}
@@ -1151,7 +1124,7 @@ void Unit::choose_best_attack_mode(int attackDistance, char targetMobileType)
 					(attackInfoChecking->attack_range==attackInfoMaxRange->attack_range &&
 					 attackInfoChecking->attack_damage>attackInfoMaxRange->attack_damage)))
 				{
-					maxAttackRangeMode = UCHAR(i);
+					maxAttackRangeMode = i;
 					attackInfoMaxRange = attackInfoChecking;
 				}
 			}
@@ -1186,10 +1159,10 @@ void Unit::choose_best_attack_mode(int attackDistance, char targetMobileType)
 //
 void Unit::set_attack_dir(short curX, short curY, short targetX, short targetY)
 {
-	UCHAR targetDir = get_dir(curX, curY, targetX, targetY);
+	int targetDir = get_dir(curX, curY, targetX, targetY);
 	if(unit_res[unit_id]->unit_class==UNIT_CLASS_SHIP)
 	{
-		UCHAR attackDir1, attackDir2;
+		int attackDir1, attackDir2;
 
 		attackDir1 = (targetDir+2)%MAX_SPRITE_DIR_TYPE;
 		attackDir2 = (targetDir+6)%MAX_SPRITE_DIR_TYPE;
