@@ -2,6 +2,7 @@
  * Seven Kingdoms: Ancient Adversaries
  *
  * Copyright 1997,1998 Enlight Software Ltd.
+ * Copyright 2020 Jesse Allen
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -48,26 +49,8 @@ static Button 			button_set_stop[MAX_STOP_FOR_CARAVAN];
 static Button 			button_go_stop[MAX_STOP_FOR_CARAVAN];
 static Button 			button_cancel_stop[MAX_STOP_FOR_CARAVAN];
 static ButtonCustom	button_select_array[MAX_STOP_FOR_CARAVAN][MAX_GOODS_SELECT_BUTTON];
-static char				button_select_enable_flag[MAX_STOP_FOR_CARAVAN][MAX_GOODS_SELECT_BUTTON];
 
 static void				i_disp_caravan_select_button(ButtonCustom *button, int repaintBody);
-static char				goods_num[MAX_STOP_FOR_CARAVAN];
-
-static char				dummyCaravanEnableFlag[MAX_STOP_FOR_CARAVAN][MAX_GOODS_SELECT_BUTTON];
-static char				dummyCaravanGoodsNum[MAX_STOP_FOR_CARAVAN];
-//------------- define static function ------------//
-static void update_caravan_stop_and_goods_to_dummy(UnitCaravan *caravanPtr)
-{
-	CaravanStop *stopPtr = caravanPtr->stop_array;
-	for(int i=0; i<MAX_STOP_FOR_CARAVAN; i++, stopPtr++)
-	{
-		if(!stopPtr->firm_recno)
-			continue;
-
-		err_when(firm_array.is_deleted(stopPtr->firm_recno));
-		dummyCaravanGoodsNum[i] = stopPtr->update_pick_up(dummyCaravanEnableFlag[i]);
-	}
-}
 
 //--------- Begin of function UnitCaravan::UnitCaravan ---------//
 //
@@ -92,14 +75,6 @@ UnitCaravan::UnitCaravan()
 //
 void UnitCaravan::disp_info(int refreshFlag)
 {
-	//----- for multiplayer game, skip displaying information for the first frame --------//
-	if(remote.is_enable())
-	{
-		if(unit_array.mp_first_frame_to_select_caravan && // first frame
-			unit_array.mp_pre_selected_caravan_recno==sprite_recno) // is selected
-			return;
-	}
-
 	disp_basic_info(INFO_Y1, refreshFlag);
 
 	if( !config.show_ai_info && !is_own() )
@@ -107,7 +82,7 @@ void UnitCaravan::disp_info(int refreshFlag)
 
 	disp_stop(INFO_Y1+54, refreshFlag);
 
-	disp_goods(INFO_Y1+235, refreshFlag);
+	disp_goods(INFO_Y1+234, refreshFlag);
 }
 //---------- End of function UnitCaravan::disp_info ----------//
 
@@ -122,6 +97,9 @@ void UnitCaravan::detect_info()
 	if( detect_basic_info() )
 		return;
 
+	if( detect_select_hotkey() )
+		return;
+
 	if( !is_own() && !config.show_ai_info )
 		return;
 
@@ -197,7 +175,7 @@ void UnitCaravan::disp_stop(int dispY1, int refreshFlag)
 				firmPtr = firm_array[ stop_array[i].firm_recno ];
 				nation_array[firmPtr->nation_recno]->disp_nation_color(x+4, y+4);
 				font_san.put(x+20, y+4, firmPtr->firm_name());
-				font_san.put(x+4, y+19, _("Pick up: "));
+				font_san.put(x+4, y+19, _("Pick up:"));
 
 #if(defined(FRENCH))
 				button_set_stop[i].paint_text( x+4, y+37, x+86, y+56, "Set Stop" );
@@ -256,9 +234,6 @@ void UnitCaravan::detect_stop()
 
 		for(int b=0; b<MAX_GOODS_SELECT_BUTTON; ++b)
 		{
-			if(!button_select_enable_flag[i][b])
-				continue;
-
 			if(button_select_array[i][b].detect())
 			{
 				// ###### begin Gilbert 26/9 ######//
@@ -288,16 +263,14 @@ void UnitCaravan::disp_goods_select_button(int stopNum, int dispY1, int refreshF
 	if(refreshFlag!=INFO_REPAINT && refreshFlag!=INFO_UPDATE)
 		return;
 
-	#define SHIFT_X_OFFSET	60
+	#define SHIFT_X_OFFSET	73
 	#define SELECT_BUTTON_WIDTH	16
 	#define SELECT_BUTTON_HEIGHT	16
 
 	CaravanStop	*stopPtr = &stop_array[stopNum];
+	Firm *firmPtr = firm_array[stopPtr->firm_recno];
 
-	if( !goods_num[stopNum] )
-		return;
-
-	int  x=INFO_X1+SHIFT_X_OFFSET, y=dispY1+17, x1;
+	int  x=INFO_X1+SHIFT_X_OFFSET, y=dispY1+17, x1, pick_up_goods = 0;
 	char *pickUpArray = stopPtr->pick_up_array;
 	char isPush;
 
@@ -307,9 +280,61 @@ void UnitCaravan::disp_goods_select_button(int stopNum, int dispY1, int refreshF
 
 	for(int i=1 ;i<=MAX_PICK_UP_GOODS; ++i, pickUpArray++)
 	{
+		int rawId = i;
+		if( rawId < 1 || rawId > MAX_RAW )
+			rawId = 0;
+		int productId = i-MAX_RAW;
+		if( productId < 1 || productId > MAX_PRODUCT )
+			productId = 0;
+
+		int stock = -1;
+
+		if( FirmMarket *firmMarket = firmPtr->cast_to_FirmMarket() )
+		{
+			MarketGoods *marketGoods;
+			if( rawId )
+			{
+				marketGoods = firmMarket->market_raw_array[rawId-1];
+				err_when( marketGoods && marketGoods->raw_id != rawId );
+			}
+			else if( productId )
+			{
+				marketGoods = firmMarket->market_product_array[productId-1];
+				err_when( marketGoods && marketGoods->product_raw_id != productId );
+			}
+			else
+			{
+				err_here();
+				marketGoods = NULL;
+			}
+
+			if( marketGoods )
+			{
+				stock = (int) marketGoods->stock_qty;
+			}
+		}
+		else if( FirmMine *firmMine = firmPtr->cast_to_FirmMine() )
+		{
+			if( rawId && firmMine->raw_id == rawId )
+			{
+				stock = (int) firmMine->stock_qty;
+			}
+		}
+		else if( FirmFactory *firmFactory = firmPtr->cast_to_FirmFactory() )
+		{
+			if( productId && firmFactory->product_raw_id == productId )
+			{
+				stock = (int) firmFactory->stock_qty;
+			}
+			//else if( rawId && firmFactory->product_raw_id == rawId )
+			//{
+			//	stock = (int) firmFactory->raw_stock_qty;
+			//}
+		}
+
 		x1 = x + i*SELECT_BUTTON_WIDTH;
 
-		if(button_select_enable_flag[stopNum][i])
+		if( stock >= 0 )
 		{
 			isPush = stopPtr->pick_up_array[i-1];
 			err_when(isPush && (stopPtr->pick_up_type==AUTO_PICK_UP || stopPtr->pick_up_type==NO_PICK_UP));
@@ -317,6 +342,7 @@ void UnitCaravan::disp_goods_select_button(int stopNum, int dispY1, int refreshF
 			button_select_array[stopNum][i].paint(x1, y, x1+SELECT_BUTTON_WIDTH,
 				y+SELECT_BUTTON_HEIGHT, i_disp_caravan_select_button, ButtonCustomPara(this, i),
 				0, isPush); // 0 for inelastic
+			pick_up_goods++;
 		}
 		else
 		{
@@ -326,17 +352,15 @@ void UnitCaravan::disp_goods_select_button(int stopNum, int dispY1, int refreshF
 
 	//---------------- draw the buttons for auto_pick_up and no_pick_up -------------//
 
-	if(goods_num[stopNum]>1)
+	if( pick_up_goods>1 )
 	{
 		x1 = x;
 		isPush = (stopPtr->pick_up_type==AUTO_PICK_UP);
-		button_select_enable_flag[stopNum][AUTO_PICK_UP] = 1;
 		button_select_array[stopNum][AUTO_PICK_UP].paint(x1, y, x1+SELECT_BUTTON_WIDTH,
 			y+SELECT_BUTTON_HEIGHT, i_disp_caravan_select_button, ButtonCustomPara(this, AUTO_PICK_UP),
 			0, isPush); // 0 for inelastic
 
 		x1 = x+SELECT_BUTTON_WIDTH*NO_PICK_UP;
-		button_select_enable_flag[stopNum][NO_PICK_UP] = 1;
 		button_select_array[stopNum][NO_PICK_UP].paint(x1, y, x1+SELECT_BUTTON_WIDTH,
 			y+SELECT_BUTTON_HEIGHT, i_disp_caravan_select_button, ButtonCustomPara(this, NO_PICK_UP));
 	}
@@ -364,7 +388,6 @@ void UnitCaravan::disp_goods_select_button(int stopNum, int dispY1, int refreshF
 //
 void UnitCaravan::set_stop_pick_up(int stopId, int newPickUpType, int remoteAction)
 {
-	int remoteEnable = 0;
 	if(remote.is_enable())
 	{
 		if(!remoteAction)
@@ -430,45 +453,27 @@ void UnitCaravan::set_stop_pick_up(int stopId, int newPickUpType, int remoteActi
 			//-*******************************************************-//
 			#endif
 		}
-
-		remoteEnable = 1;
 	}
 
 	switch(newPickUpType)
 	{
-		case AUTO_PICK_UP:
-				if(remoteEnable)
-				{
-					update_caravan_stop_and_goods_to_dummy(this);
-					stop_array[stopId-1].mp_pick_up_set_auto(dummyCaravanEnableFlag[stopId-1]);
-				}
-				else
-					stop_array[stopId-1].pick_up_set_auto();
-				break;
+	case AUTO_PICK_UP:
+		stop_array[stopId-1].pick_up_set_auto();
+		break;
 
-		case NO_PICK_UP:
-				// if(remoteEnable)
-				if(remoteEnable)
-				{
-					update_caravan_stop_and_goods_to_dummy(this);
-					stop_array[stopId-1].mp_pick_up_set_none(dummyCaravanEnableFlag[stopId-1]);
-				}
-				else
-					stop_array[stopId-1].pick_up_set_none();
-				break;
+	case NO_PICK_UP:
+		stop_array[stopId-1].pick_up_set_none();
+		break;
 
-		default:
-				err_when(newPickUpType<PICK_UP_RAW_FIRST || newPickUpType>PICK_UP_PRODUCT_LAST);
-				if(remoteEnable)
-					stop_array[stopId-1].mp_pick_up_toggle(newPickUpType);
-				else
-					stop_array[stopId-1].pick_up_toggle(newPickUpType);
-				break;
+	default:
+		err_when(newPickUpType<PICK_UP_RAW_FIRST || newPickUpType>PICK_UP_PRODUCT_LAST);
+		stop_array[stopId-1].pick_up_toggle(newPickUpType);
+		break;
 	}
 
 	if(unit_array.selected_recno==sprite_recno)
 	{
-		if(!remote.is_enable() || nation_recno==nation_array.player_recno || config.show_ai_info)
+		if(nation_recno==nation_array.player_recno || config.show_ai_info)
 			disp_stop(INFO_Y1+54, INFO_UPDATE);
 	}
 }
@@ -569,9 +574,6 @@ void UnitCaravan::set_stop(int stopId, int stopXLoc, int stopYLoc, char remoteAc
 	//-------------- reset ignore_power_nation -------------//
 	ignore_power_nation = 0;
 
-	memset(button_select_enable_flag[stopId-1], 0, sizeof(char)*MAX_GOODS_SELECT_BUTTON);
-	goods_num[stopId-1] = 0;
-
 	short oldStopFirmRecno = dest_stop_id ? stop_array[dest_stop_id-1].firm_recno : 0;
 	short newStopFirmRecno;
 	memset(stopPtr->pick_up_array, 0, sizeof(char)*MAX_PICK_UP_GOODS);
@@ -643,7 +645,10 @@ void UnitCaravan::set_stop(int stopId, int stopXLoc, int stopYLoc, char remoteAc
 	//-------------------------------------------------------//
 	update_stop_list();
 
-	if(dest_stop_id)
+	//-------------------------------------------------------//
+	// handle if current stop changed when mobile
+	//-------------------------------------------------------//
+	if(dest_stop_id && journey_status!=INSIDE_FIRM)
 	{
 		if((newStopFirmRecno=stop_array[dest_stop_id-1].firm_recno) != oldStopFirmRecno)
 		{
@@ -653,12 +658,12 @@ void UnitCaravan::set_stop(int stopId, int stopXLoc, int stopYLoc, char remoteAc
 			journey_status = ON_WAY_TO_FIRM;
 		}
 	}
-	else
+	else if(journey_status!=INSIDE_FIRM)
 		stop2();
 
 	if( unit_array.selected_recno == sprite_recno )
 	{
-		if(!remote.is_enable() || nation_recno==nation_array.player_recno || config.show_ai_info)
+		if(nation_recno==nation_array.player_recno || config.show_ai_info)
 			info.disp();
 	}
 }
@@ -668,8 +673,6 @@ void UnitCaravan::set_stop(int stopId, int stopXLoc, int stopYLoc, char remoteAc
 //--------- Begin of function UnitCaravan::del_stop ---------//
 void UnitCaravan::del_stop(int stopId, char remoteAction)
 {
-	err_when(action_para || action_para2);
-
 	if(!remoteAction && remote.is_enable())
 	{
 		// packet structure : <unit recno> <stop id>
@@ -858,7 +861,8 @@ void UnitCaravan::update_stop_list()
 	if(!ourFirmExist) // none of the markets belong to our nation
 	{
 		memset(stop_array, 0, MAX_STOP_FOR_CARAVAN * sizeof(CaravanStop));
-		journey_status		= ON_WAY_TO_FIRM;
+		if(journey_status != INSIDE_FIRM)
+			journey_status = ON_WAY_TO_FIRM;
 		dest_stop_id		= 0;
 		stop_defined_num	= 0;
 		return;
@@ -885,7 +889,7 @@ void UnitCaravan::update_stop_list()
 
 			if(dist<minDist)
 			{
-				dist = minDist;
+				minDist = dist;
 				dest_stop_id = i+1;
 			}
 		}
@@ -896,22 +900,6 @@ void UnitCaravan::update_stop_list()
 //----------- End of function UnitCaravan::update_stop_list -----------//
 
 
-//--------- Begin of function UnitCaravan::update_stop_and_goods_info ---------//
-void UnitCaravan::update_stop_and_goods_info()
-{
-	update_stop_list();
-	update_caravan_stop_and_goods_to_dummy(this);
-
-	if(sprite_recno==unit_array.selected_recno && ((nation_array.player_recno && nation_recno==nation_array.player_recno) ||
-		config.show_ai_info))
-	{
-		memcpy(goods_num, dummyCaravanGoodsNum, sizeof(char)*MAX_STOP_FOR_CARAVAN);
-		memcpy(button_select_enable_flag, dummyCaravanEnableFlag, sizeof(char)*MAX_STOP_FOR_CARAVAN*MAX_GOODS_SELECT_BUTTON);
-	}
-}
-//----------- End of function UnitCaravan::update_stop_and_goods_info -----------//
-
-
 //--------- Begin of function UnitCaravan::can_set_stop ---------//
 //
 // Whether can set a caravan's stop on the given firm.
@@ -1024,6 +1012,15 @@ void UnitCaravan::pre_process()
 
 	err_when(action_mode==ACTION_DIE || cur_action==SPRITE_DIE || hit_points<=0);
 
+	//-----------------------------------------------------------------------------//
+	// process when in firm
+	//-----------------------------------------------------------------------------//
+	if(journey_status==INSIDE_FIRM)
+	{
+		caravan_in_firm();
+		return;
+	}
+
 	//-----------------------------------------------------------------------------//
 	// stop action if no stop is defined
 	//-----------------------------------------------------------------------------//
@@ -1114,10 +1111,7 @@ void UnitCaravan::pre_process()
 	//-----------------------------------------------------------------------------//
 	err_when(stop_defined_num<=1);
 
-	if(journey_status==INSIDE_FIRM)
-		caravan_in_firm();
-	else
-		caravan_on_way();
+	caravan_on_way();
 }
 //----------- End of function UnitCaravan::pre_process -----------//
 
@@ -1240,7 +1234,7 @@ void UnitCaravan::caravan_on_way()
 
 		if(stop_defined_num) // move to next stop
 		{
-			Firm *firmPtr = firm_array[stop_array[stop_defined_num-1].firm_recno];
+			Firm *firmPtr = firm_array[stop_array[dest_stop_id-1].firm_recno];
 			move_to_firm_surround(firmPtr->loc_x1, firmPtr->loc_y1, sprite_info->loc_width, sprite_info->loc_height, firmPtr->firm_id);
 		}
 		return;
@@ -1257,21 +1251,6 @@ void UnitCaravan::caravan_on_way()
 		  (nextXLoc>=firmPtr->loc_x1-1 && nextXLoc<=firmPtr->loc_x2+1 &&
 			nextYLoc>=firmPtr->loc_y1-1 && nextYLoc<=firmPtr->loc_y2+1) )) // in the surrounding of the firm
 	{
-		//-*************************************************************-//
-		/*if(nation_recno==nation_array.player_recno)
-		{
-			OutputDebugString("surround firm\r\n");
-			for(int type = 0; type<MAX_GOODS_SELECT_BUTTON; type++)
-			{
-				if(button_select_enable_flag[0][type])
-					break;
-			}
-
-			stop_array[0].pick_up_array[type] ^= 1;
-			set_stop_pick_up(1, type, COMMAND_PLAYER);
-		}*/
-		//-*************************************************************-//
-
 		//-------------------- update pick_up_array --------------------//
 		stopPtr->update_pick_up();
 
@@ -1596,3 +1575,66 @@ int UnitCaravan::carrying_qty(int pickUpType)
 //---------- End of function UnitCaravan::carrying_qty ----------//
 
 
+//--------- Begin of function UnitCaravan::copy_route ---------//
+//
+// Copies trade route from copyUnitRecno to this caravan.
+//
+void UnitCaravan::copy_route(short copyUnitRecno, int remoteAction)
+{
+	if( sprite_recno == copyUnitRecno )
+		return;
+
+	UnitCaravan* copyUnit = (UnitCaravan*)unit_array[copyUnitRecno];
+
+	if( copyUnit->nation_recno != nation_recno )
+		return;
+
+	if( remote.is_enable() && !remoteAction )
+	{
+		// packet structure : <unit recno> <copy recno>
+		short *shortPtr = (short *)remote.new_send_queue_msg(MSG_U_CARA_COPY_ROUTE, 2*sizeof(short));
+		*shortPtr = sprite_recno;
+
+		shortPtr[1] = copyUnitRecno;
+		return;
+	}
+
+	// clear existing stops
+	int num_stops = stop_defined_num;
+	for( int i=0; i<num_stops; i++ )
+		del_stop(1, COMMAND_AUTO); // stop ids shift up
+
+	CaravanStop* caravanStopA = copyUnit->stop_array;
+	CaravanStop* caravanStopB = stop_array;
+	for( int i=0; i<MAX_STOP_FOR_CARAVAN; i++, caravanStopA++, caravanStopB++ )
+	{
+		if( !caravanStopA->firm_recno )
+			break;
+
+		if( firm_array.is_deleted(caravanStopA->firm_recno) )
+			continue;
+
+		Firm* firmPtr = firm_array[caravanStopA->firm_recno];
+		set_stop(i+1, caravanStopA->firm_loc_x1, caravanStopA->firm_loc_y1, COMMAND_AUTO);
+
+		if( caravanStopA->pick_up_type == AUTO_PICK_UP )
+		{
+			set_stop_pick_up(i+1, AUTO_PICK_UP, COMMAND_AUTO );
+		}
+
+		else if( caravanStopA->pick_up_type == NO_PICK_UP )
+		{
+			set_stop_pick_up(i+1, NO_PICK_UP, COMMAND_AUTO );
+		}
+
+		else
+		{
+			for( int b=0; b<MAX_PICK_UP_GOODS; ++b )
+			{
+				if( caravanStopA->pick_up_array[b] != caravanStopB->pick_up_array[b] )
+					set_stop_pick_up(i+1, b+1, COMMAND_PLAYER);
+			}
+		}
+	}
+}
+//---------- End of function UnitCaravan::copy_route ----------//
