@@ -31,97 +31,19 @@
 
 DBGLOG_DEFAULT_CHANNEL(MultiPlayer);
 
-#define MP_UDP_MAX_PACKET_SIZE 800
+#define MP_RECV_BUFFER_SIZE 512*3
+#define MP_RECV_MAX_BUFFER_SIZE 512*20
 
 const uint16_t UDP_GAME_PORT = 19255;
 const uint16_t UDP_MONITOR_PORT = 19383;
 
-SessionDesc::SessionDesc()
-{
-	misc.uuid_clear(id);
-	flags = SESSION_PREGAME;
-	max_players = MAX_NATION;
-	player_count = 1;
-	memset(password, 0, sizeof(session_name));
-	memset(password, 0, sizeof(password));
-	this->address.host = ENET_HOST_ANY;
-	this->address.port = UDP_GAME_PORT;
-}
-
-SessionDesc::SessionDesc(const char *name, const char *pass, ENetAddress *address)
-{
-	misc.uuid_generate_random(id); // Assume LAN until command line improved
-	flags = SESSION_PREGAME;
-	max_players = MAX_NATION;
-	player_count = 1;
-	strncpy(session_name, name, MP_FRIENDLY_NAME_LEN);
-	session_name[MP_FRIENDLY_NAME_LEN] = 0;
-	strncpy(password, pass, MP_FRIENDLY_NAME_LEN);
-	password[MP_FRIENDLY_NAME_LEN] = 0;
-	if (pass[0])
-		flags |= SESSION_PASSWORD;
-	this->address.host = address->host;
-	this->address.port = address->port;
-}
-
-SessionDesc::SessionDesc(MpMsgUserSessionStatus *m, ENetAddress *address)
-{
-	misc.uuid_copy(id, m->session_id);
-	flags = m->flags;
-	max_players = MAX_NATION;
-	player_count = 1;
-	strncpy(this->session_name, m->session_name, MP_FRIENDLY_NAME_LEN);
-	session_name[MP_FRIENDLY_NAME_LEN] = 0;
-	memset(password, 0, sizeof(password));
-	this->address.host = address->host;
-	this->address.port = address->port;
-}
-
-SessionDesc::SessionDesc(MpMsgSession *m)
-{
-	misc.uuid_copy(id, m->session_id);
-	flags = m->flags;
-	max_players = MAX_NATION;
-	player_count = 1;
-	strncpy(session_name, m->session_name, MP_FRIENDLY_NAME_LEN);
-	session_name[MP_FRIENDLY_NAME_LEN] = 0;
-	memset(password, 0, sizeof(password));
-	this->address.host = ENET_HOST_ANY;
-	this->address.port = 0;
-}
-
-SessionDesc::SessionDesc(const SessionDesc &src)
-{
-	misc.uuid_copy(id, src.id);
-	flags = src.flags;
-	max_players = src.max_players;
-	player_count = src.player_count;
-	strncpy(session_name, src.session_name, MP_FRIENDLY_NAME_LEN);
-	session_name[MP_FRIENDLY_NAME_LEN] = 0;
-	strncpy(password, src.password, MP_FRIENDLY_NAME_LEN);
-	password[MP_FRIENDLY_NAME_LEN] = 0;
-	address.host = src.address.host;
-	address.port = src.address.port;
-}
-
-SessionDesc& SessionDesc::operator= (const SessionDesc &src)
-{
-	misc.uuid_copy(id, src.id);
-	flags = src.flags;
-	max_players = src.max_players;
-	player_count = src.player_count;
-	strncpy(session_name, src.session_name, MP_FRIENDLY_NAME_LEN);
-	session_name[MP_FRIENDLY_NAME_LEN] = 0;
-	strncpy(password, src.password, MP_FRIENDLY_NAME_LEN);
-	password[MP_FRIENDLY_NAME_LEN] = 0;
-	address.host = src.address.host;
-	address.port = src.address.port;
-	return *this;
-}
-
 inline bool cmp_addr(ENetAddress *a, ENetAddress *b)
 {
+#ifdef USE_ENET
 	return a->host == b->host && a->port == b->port;
+#else
+	return 0;
+#endif
 }
 
 // to start a multiplayer game, first check if it is called from a
@@ -133,14 +55,26 @@ inline bool cmp_addr(ENetAddress *a, ENetAddress *b)
 // user to select, call init and pass the guid of the selected
 // service; create_session or poll_sessions+join_session;
 
+namespace {
+	ENetAddress empty_address()
+	{
+		ENetAddress address;
+#ifdef USE_ENET
+		address.host = ENET_HOST_ANY;
+		address.port = ENET_PORT_ANY;
+#endif
+		return address;
+	}
+}
+
 MultiPlayer::MultiPlayer() :
-	current_sessions(sizeof(SessionDesc), 10)
+	current_sessions(sizeof(SessionDesc), 10),
+	my_player("?my_player?", empty_address())
 {
 	init_flag = 0;
 	lobbied_flag = 0;
 	supported_protocols = TCPIP;
 	my_player_id = 0;
-	my_player = NULL;
 	joined_session.flags = 0;
 	recv_buf = NULL;
 }
@@ -152,12 +86,12 @@ MultiPlayer::~MultiPlayer()
 
 void MultiPlayer::init(ProtocolType protocol_type)
 {
+#ifdef USE_ENET
 	if (init_flag)
 		return;
 
 	lobbied_flag = 0;
 	my_player_id = 0;
-	my_player = NULL;
 	service_provider.host = ENET_HOST_ANY;
 	misc.uuid_clear(service_login_id);
 	update_available = -1;
@@ -183,12 +117,15 @@ void MultiPlayer::init(ProtocolType protocol_type)
 	}
 
 	recv_buf = new char[MP_RECV_BUFFER_SIZE];
+	recv_buffer_size = MP_RECV_BUFFER_SIZE;
 
 	init_flag = 1;
+#endif
 }
 
 void MultiPlayer::deinit()
 {
+#ifdef USE_ENET
 	if (!init_flag) {
 		return;
 	}
@@ -217,10 +154,6 @@ void MultiPlayer::deinit()
 			delete pending_pool[i];
 		}
 	}
-	if (my_player) {
-		delete my_player;
-		my_player = NULL;
-	}
 
 	close_port();
 	enet_deinitialize();
@@ -228,6 +161,7 @@ void MultiPlayer::deinit()
 	if (recv_buf) {
 		delete [] recv_buf;
 	}
+#endif
 }
 
 // init_lobbied
@@ -235,26 +169,31 @@ void MultiPlayer::deinit()
 // Returns non-zero on success.
 int MultiPlayer::init_lobbied(int maxPlayers, char *cmdLine)
 {
+#ifdef USE_ENET
 	MSG("Launching a multiplayer game from command line maxPlayers=%d, cmdLine='%s'\n", maxPlayers, cmdLine);
 
 	if (cmdLine) {
 		ENetAddress address;
-		SessionDesc *session;
 
 		if (enet_address_set_host(&address, cmdLine) != 0) {
 			return 0;
 		}
 		address.port = UDP_GAME_PORT;
 
-		session = new SessionDesc("Lobbied Game", "1", &address);
+		guuid_t sessionId;
+		misc.uuid_generate_random(sessionId);
+		SessionDesc session("Lobbied Game", sessionId, SessionFlags::Pregame, address);
+		strcpy(session.password, "1");
+		session.max_players = maxPlayers;
 
-		current_sessions.linkin(session);
+		current_sessions.linkin(&session);
 
 		lobbied_flag = 2;
 	} else {
 		// hosting doesn't work yet
 		lobbied_flag = 1;
 	}
+#endif
 
 	return 1;
 }
@@ -289,6 +228,7 @@ int MultiPlayer::is_update_available()
 ENetSocket MultiPlayer::create_socket(uint16_t port)
 {
 	ENetSocket socket;
+#ifdef USE_ENET
 	ENetAddress address;
 
 	socket = enet_socket_create(ENET_SOCKET_TYPE_DATAGRAM);
@@ -305,13 +245,16 @@ ENetSocket MultiPlayer::create_socket(uint16_t port)
 	}
 	enet_socket_set_option(socket, ENET_SOCKOPT_NONBLOCK, 1);
 
+#endif
 	return socket;
 }
 
 void MultiPlayer::destroy_socket(ENetSocket socket)
 {
+#ifdef USE_ENET
 	if (socket != ENET_SOCKET_NULL)
 		enet_socket_destroy(socket);
+#endif
 }
 
 // Open a multiplayer game port to enable the network service. If successful,
@@ -319,6 +262,7 @@ void MultiPlayer::destroy_socket(ENetSocket socket)
 // try a second time using any port.
 int MultiPlayer::open_port(uint16_t port, int fallback)
 {
+#ifdef USE_ENET
 	ENetAddress address;
 
 	err_when(!init_flag);
@@ -341,21 +285,25 @@ int MultiPlayer::open_port(uint16_t port, int fallback)
 
 	if (fallback)
 		return open_port(0, 0);
+#endif
 
 	return 0;
 }
 
 void MultiPlayer::close_port()
 {
+#ifdef USE_ENET
 	if (host) {
 		enet_host_destroy(host);
 		host = NULL;
 	}
+#endif
 }
 
 int MultiPlayer::set_service_provider(const char *host)
 {
-	if (host) {
+#ifdef USE_ENET
+	if (host && *host) {
 		enet_address_set_host(&service_provider, host);
 		service_provider.port = UDP_MONITOR_PORT;
 	} else {
@@ -363,10 +311,13 @@ int MultiPlayer::set_service_provider(const char *host)
 	}
 
 	return service_provider.host != ENET_HOST_ANY;
+#endif
+	return 0;
 }
 
 int MultiPlayer::poll_sessions()
 {
+#ifdef USE_ENET
 	ENetAddress a;
 	ENetBuffer b;
 	int ret;
@@ -379,7 +330,7 @@ int MultiPlayer::poll_sessions()
 	}
 
 	b.data = recv_buf;
-	b.dataLength = MP_RECV_BUFFER_SIZE;
+	b.dataLength = recv_buffer_size;
 
 	ret = MP_POLL_NO_UPDATE;
 	login_failed = 0;
@@ -390,12 +341,12 @@ int MultiPlayer::poll_sessions()
 		case MPMSG_USER_SESSION_STATUS: {
 			MpMsgUserSessionStatus *m = (MpMsgUserSessionStatus *)recv_buf;
 
-			if ((m->flags & (SESSION_HOSTING|SESSION_PREGAME)) == 0)
+			if ((m->flags & (SessionFlags::Hosting|SessionFlags::Pregame)) == 0)
 				break;
 			if (get_session(&a))
 				break;
 
-			SessionDesc desc(m, &a);
+			SessionDesc desc(m->session_name, m->session_id, m->flags, a);
 			current_sessions.linkin(&desc);
 
 			break;
@@ -422,7 +373,7 @@ int MultiPlayer::poll_sessions()
 					break;
 				}
 
-				SessionDesc desc(m);
+				SessionDesc desc(m->session_name, m->session_id, m->flags, empty_address());
 				current_sessions.linkin(&desc);
 			}
 
@@ -442,6 +393,8 @@ int MultiPlayer::poll_sessions()
 	}
 
 	return ret;
+#endif
+	return 0;
 }
 
 // return a session description
@@ -471,7 +424,7 @@ SessionDesc *MultiPlayer::get_session(guuid_t id)
 	int i;
 	for (i = 1; i <= current_sessions.size(); i++) {
 		SessionDesc *p = (SessionDesc *)current_sessions.get(i);
-		if (p && !misc.uuid_compare(p->id, id))
+		if (p && !misc.uuid_compare(p->session_id, id))
 			return p;
 	}
 	return NULL;
@@ -490,6 +443,7 @@ SessionDesc *MultiPlayer::get_current_session()
 // return 1 if success
 int MultiPlayer::create_session(char *sessionName, char *password, int maxPlayers)
 {
+#ifdef USE_ENET
 	const char *name;
 
 	err_when(!init_flag || maxPlayers <= 0 || maxPlayers > MAX_NATION || host);
@@ -511,26 +465,28 @@ int MultiPlayer::create_session(char *sessionName, char *password, int maxPlayer
 	strcpy(joined_session.password, password);
 	joined_session.max_players = maxPlayers;
 	joined_session.player_count = 1;
-	joined_session.flags |= SESSION_HOSTING | SESSION_PREGAME;
+	joined_session.flags |= SessionFlags::Hosting | SessionFlags::Pregame;
 	if (password[0])
-		joined_session.flags |= SESSION_PASSWORD;
+		joined_session.flags |= SessionFlags::Password;
 	if (joined_session.player_count >= joined_session.max_players)
-		joined_session.flags |= SESSION_FULL;
+		joined_session.flags |= SessionFlags::Full;
 	if (service_provider.host != ENET_HOST_ANY)
-		misc.uuid_clear(joined_session.id); // generated by service
+		misc.uuid_clear(joined_session.session_id); // generated by service
 	else
-		misc.uuid_generate_random(joined_session.id); // generated for LAN only
+		misc.uuid_generate_random(joined_session.session_id); // generated for LAN only
 	joined_session.address.host = host->address.host;
 	joined_session.address.port = host->address.port;
 
 	set_my_player_id(1);
-	player_pool[0] = my_player; // can we skip polling players?
+	player_pool[0] = &my_player; // can we skip polling players?
+#endif
 
 	return 1;
 }
 
 int MultiPlayer::connect_host()
 {
+#ifdef USE_ENET
 	ENetPeer *peer;
 	PlayerDesc *game_host;
 
@@ -545,10 +501,11 @@ int MultiPlayer::connect_host()
 	if (!peer)
 		return 0;
 
-	game_host = new PlayerDesc(&joined_session.address);
+	game_host = new PlayerDesc("?Anonymous?", joined_session.address);
 	game_host->id = 1;
 	game_host->authorized = 1;
 	peer->data = game_host;
+#endif
 
 	return 1;
 }
@@ -558,6 +515,7 @@ int MultiPlayer::connect_host()
 //
 int MultiPlayer::join_session(SessionDesc *session)
 {
+#ifdef USE_ENET
 	err_when(!session || host);
 
 	if (!open_port(0, 0)) {
@@ -566,8 +524,9 @@ int MultiPlayer::join_session(SessionDesc *session)
 	}
 
 	joined_session = *session;
-	joined_session.flags &= ~SESSION_HOSTING; // we're not the host
+	joined_session.flags &= ~SessionFlags::Hosting; // we're not the host
 	connect_host();
+#endif
 
 	return 1;
 }
@@ -576,6 +535,7 @@ int MultiPlayer::join_session(SessionDesc *session)
 // disconnect procedures started.
 int MultiPlayer::close_session()
 {
+#ifdef USE_ENET
 	ENetPeer *peer;
 	int count;
 
@@ -592,18 +552,23 @@ int MultiPlayer::close_session()
 	}
 
 	return count;
+#endif
+	return 0;
 }
 
 void MultiPlayer::game_starting()
 {
+#ifdef USE_ENET
 	packet_mode = ENET_PACKET_FLAG_UNSEQUENCED;
+#endif
 }
 
 void MultiPlayer::disable_new_connections()
 {
+#ifdef USE_ENET
 	ENetPeer *peer;
 
-	joined_session.flags &= ~SESSION_PREGAME;
+	joined_session.flags &= ~SessionFlags::Pregame;
 
 	for (peer = host->peers; peer < &host->peers[host->peerCount]; ++peer) {
 		if (peer->data) {
@@ -612,6 +577,7 @@ void MultiPlayer::disable_new_connections()
 				enet_peer_disconnect(peer, 0);
 		}
 	}
+#endif
 }
 
 // Returns the next available player id. Used by the game host for adding players.
@@ -702,6 +668,7 @@ PlayerDesc *MultiPlayer::yank_pending_player(ENetAddress *address)
 // Returns 0 when the player was not added.
 int MultiPlayer::add_player(uint32_t playerId, char *name, ENetAddress *address, char contact)
 {
+#ifdef USE_ENET
 	PlayerDesc *player = NULL;
 	ENetPeer *peer;
 
@@ -716,12 +683,14 @@ int MultiPlayer::add_player(uint32_t playerId, char *name, ENetAddress *address,
 		player = (PlayerDesc *)peer->data;
 	}
 	if (!player) {
-		player = new PlayerDesc(address);
+		player = new PlayerDesc(name, *address);
+	}
+	else {
+		strncpy(player->name, name, MP_FRIENDLY_NAME_LEN);
+		player->name[MP_FRIENDLY_NAME_LEN] = 0;
 	}
 
 	player->id = playerId;
-	strncpy(player->name, name, MP_FRIENDLY_NAME_LEN);
-	player->name[MP_FRIENDLY_NAME_LEN] = 0;
 	player->authorized = 1;
 
 	if (peer) {
@@ -752,6 +721,7 @@ int MultiPlayer::add_player(uint32_t playerId, char *name, ENetAddress *address,
 	}
 
 	MSG("Player '%s' (%d) recognized.\n", player->name, playerId);
+#endif
 
 	return 1;
 }
@@ -763,11 +733,12 @@ int MultiPlayer::add_player(uint32_t playerId, char *name, ENetAddress *address,
 // Returns 0 when a player is not authorized.
 int MultiPlayer::auth_player(uint32_t playerId, char *name, char *password)
 {
+#ifdef USE_ENET
 	ENetPeer *peer;
 	PlayerDesc *player;
 
 	err_when(!host);
-	err_when(!(joined_session.flags & SESSION_HOSTING));
+	err_when(!(joined_session.flags & SessionFlags::Hosting));
 
 	peer = get_peer(playerId);
 	if (!peer || !peer->data) {
@@ -775,7 +746,7 @@ int MultiPlayer::auth_player(uint32_t playerId, char *name, char *password)
 	}
 	player = (PlayerDesc *)peer->data;
 
-	if ((joined_session.flags & SESSION_PASSWORD) && memcmp(password, joined_session.password, MP_FRIENDLY_NAME_LEN)) {
+	if ((joined_session.flags & SessionFlags::Password) && memcmp(password, joined_session.password, MP_FRIENDLY_NAME_LEN)) {
 		MSG("Player (%d) password is incorrect.\n", playerId);
 		return 0;
 	}
@@ -786,6 +757,7 @@ int MultiPlayer::auth_player(uint32_t playerId, char *name, char *password)
 
 	MSG("Player '%s' (%d) was authorized.\n", player->name, playerId);
 	update_player_pool();
+#endif
 
 	return 1;
 }
@@ -797,17 +769,15 @@ void MultiPlayer::create_my_player(char *playerName)
 		name = playerName;
 	else
 		name = "?Anonymous?";
-	if (my_player)
-		strcpy(my_player->name, name);
-	else
-		my_player = new PlayerDesc(name);
+
+	my_player = PlayerDesc(name, empty_address());
 }
 
 int MultiPlayer::set_my_player_id(uint32_t playerId)
 {
 	my_player_id = playerId;
-	my_player->id = playerId;
-	my_player->authorized = 1;
+	my_player.id = playerId;
+	my_player.authorized = 1;
 
 	MSG("You have been assigned id=%d\n", playerId);
 
@@ -817,6 +787,7 @@ int MultiPlayer::set_my_player_id(uint32_t playerId)
 // Deletes a player by id, disconnecting if necessary.
 void MultiPlayer::delete_player(uint32_t playerId)
 {
+#ifdef USE_ENET
 	ENetPeer *peer;
 	PlayerDesc *player;
 
@@ -838,6 +809,7 @@ void MultiPlayer::delete_player(uint32_t playerId)
 
 	MSG("Player '%s' (%d) deleted.\n", player->name, playerId);
 	delete player;
+#endif
 }
 
 static int sort_players(const void *a, const void *b)
@@ -851,6 +823,7 @@ static int sort_players(const void *a, const void *b)
 
 int MultiPlayer::poll_players()
 {
+#ifdef USE_ENET
 	static unsigned long last_broadcast_time;
 	static int poll_time;
 	unsigned long current_time;
@@ -873,8 +846,8 @@ int MultiPlayer::poll_players()
 			// communicate with service provider
 			ENetBuffer b;
 
-			if (joined_session.flags & SESSION_HOSTING) {
-				if (misc.uuid_is_null(joined_session.id)) {
+			if (joined_session.flags & SessionFlags::Hosting) {
+				if (misc.uuid_is_null(joined_session.session_id)) {
 					send_req_session_id();
 					poll_time = 300;
 					ret = MP_POLL_LOGIN_PENDING;
@@ -888,7 +861,7 @@ int MultiPlayer::poll_players()
 			}
 
 			b.data = recv_buf;
-			b.dataLength = MP_RECV_BUFFER_SIZE;
+			b.dataLength = recv_buffer_size;
 
 			while (enet_socket_receive(session_monitor, &a, &b, 1)>0) {
 				uint32_t id = *(uint32_t *)recv_buf;
@@ -912,7 +885,7 @@ int MultiPlayer::poll_players()
 					if (a.host != service_provider.host)
 						break;
 
-					misc.uuid_copy(joined_session.id, m->session_id);
+					misc.uuid_copy(joined_session.session_id, m->session_id);
 					break;
 					}
 				case MPMSG_SESSION_ADDR: {
@@ -926,7 +899,7 @@ int MultiPlayer::poll_players()
 					}
 					if (a.host != service_provider.host)
 						break;
-					if (misc.uuid_compare(joined_session.id, m->session_id))
+					if (misc.uuid_compare(joined_session.session_id, m->session_id))
 						break;
 
 					joined_session.address.host = m->host;
@@ -940,7 +913,7 @@ int MultiPlayer::poll_players()
 
 					if (a.host != service_provider.host)
 						break;
-					if (!(joined_session.flags & SESSION_HOSTING))
+					if (!(joined_session.flags & SessionFlags::Hosting))
 						break;
 
 					do_host_nat_punch(m);
@@ -965,14 +938,17 @@ int MultiPlayer::poll_players()
 	}
 
 	return ret;
+#endif
+	return 0;
 }
 
 void MultiPlayer::update_player_pool()
 {
+#ifdef USE_ENET
 	ENetPeer *peer;
 	int i;
 
-	player_pool[0] = my_player;
+	player_pool[0] = &my_player;
 	i = 1;
 	for (peer = host->peers; peer < &host->peers[host->peerCount]; ++peer) {
 		if (i >= joined_session.max_players)
@@ -988,14 +964,15 @@ void MultiPlayer::update_player_pool()
 
 	joined_session.player_count = i;
 	if (joined_session.player_count >= joined_session.max_players)
-		joined_session.flags |= SESSION_FULL;
+		joined_session.flags |= SessionFlags::Full;
 	else
-		joined_session.flags &= ~SESSION_FULL;
+		joined_session.flags &= ~SessionFlags::Full;
 
 	for ( ; i < MAX_NATION; i++)
 		player_pool[i] = NULL;
 
 	qsort(&player_pool, joined_session.player_count, sizeof(player_pool[0]), &sort_players);
+#endif
 }
 
 // retrieve the ith-1 index from the player pool array
@@ -1026,6 +1003,7 @@ PlayerDesc *MultiPlayer::search_player(uint32_t playerId)
 // retrieve the player by ID value from enet peer arrary
 ENetPeer *MultiPlayer::get_peer(uint32_t playerId)
 {
+#ifdef USE_ENET
 	ENetPeer *peer;
 
 	err_when(!host);
@@ -1044,6 +1022,7 @@ ENetPeer *MultiPlayer::get_peer(uint32_t playerId)
 	if (peer < &host->peers[host->peerCount]) {
 		return peer;
 	}
+#endif
 
 	return NULL;
 }
@@ -1051,6 +1030,7 @@ ENetPeer *MultiPlayer::get_peer(uint32_t playerId)
 // retrieve the player by ID value from enet peer arrary
 ENetPeer *MultiPlayer::get_peer(ENetAddress *address)
 {
+#ifdef USE_ENET
 	ENetPeer *peer;
 
 	err_when(!host);
@@ -1063,6 +1043,7 @@ ENetPeer *MultiPlayer::get_peer(ENetAddress *address)
 	if (peer < &host->peers[host->peerCount]) {
 		return peer;
 	}
+#endif
 
 	return NULL;
 }
@@ -1075,6 +1056,7 @@ ENetPeer *MultiPlayer::get_peer(ENetAddress *address)
 //
 int MultiPlayer::is_player_connecting(uint32_t playerId)
 {
+#ifdef USE_ENET
 	ENetPeer *peer;
 
 	if (playerId == my_player_id)
@@ -1085,6 +1067,8 @@ int MultiPlayer::is_player_connecting(uint32_t playerId)
 		return 0;
 
 	return peer->state == ENET_PEER_STATE_CONNECTED;
+#endif
+	return 0;
 }
 
 int MultiPlayer::get_player_count()
@@ -1100,19 +1084,15 @@ int MultiPlayer::get_player_count()
 //
 int MultiPlayer::send(uint32_t to, void *data, uint32_t msg_size)
 {
+#ifdef USE_ENET
 	ENetPacket *packet;
 
-	err_when(!host);
+	err_when(!host || msg_size > MP_RECV_MAX_BUFFER_SIZE);
 
 	if (to == my_player_id) {
 		return 0;
 	}
 
-	if (msg_size > MP_UDP_MAX_PACKET_SIZE) {
-		ERR("Packet message exceeds maximum size.\n");
-		return 0;
-	}
-
 	packet = enet_packet_create(
 		data,
 		msg_size,
@@ -1130,27 +1110,33 @@ int MultiPlayer::send(uint32_t to, void *data, uint32_t msg_size)
 			}
 		}
 	} else {
-		ENetPeer *peer;
-
-		peer = get_peer(to);
-		if (!peer) {
-			return 0;
-		}
-
-		enet_peer_send(peer, 0, packet);
+		ENetPeer *peer = get_peer(to);
+		if (peer)
+			enet_peer_send(peer, 0, packet);
 	}
 
+	if( !packet->referenceCount )
+	{
+		enet_packet_destroy(packet); // free unused packet
+		return 0;
+	}
+
+	// packet is freed by enet
+	return 1;
+#endif
+
 	return 1;
 }
 
 void MultiPlayer::send_user_session_status(ENetAddress *a)
 {
+#ifdef USE_ENET
 	ENetBuffer b;
 	MpMsgUserSessionStatus m;
 
 	if (!host)
 		return;
-	if (misc.uuid_is_null(joined_session.id))
+	if (misc.uuid_is_null(joined_session.session_id))
 		return;
 
 	b.data = &m;
@@ -1162,16 +1148,18 @@ void MultiPlayer::send_user_session_status(ENetAddress *a)
 	} else {
 		misc.uuid_clear(m.login_id);
 	}
-	misc.uuid_copy(m.session_id, joined_session.id);
+	misc.uuid_copy(m.session_id, joined_session.session_id);
 	m.player_id = my_player_id;
 	m.flags = joined_session.flags;
 	strncpy(m.session_name, joined_session.session_name, MP_FRIENDLY_NAME_LEN);
 
 	enet_socket_send(host->socket, a, &b, 1);
+#endif
 }
 
 void MultiPlayer::send_req_login_id()
 {
+#ifdef USE_ENET
 	ENetBuffer b;
 	MpMsgReqLoginId m;
 
@@ -1179,13 +1167,15 @@ void MultiPlayer::send_req_login_id()
 	b.dataLength = sizeof(MpMsgReqLoginId);
 
 	m.msg_id = MPMSG_REQ_LOGIN_ID;
-	strncpy(m.name, my_player->name, MP_FRIENDLY_NAME_LEN);
+	strncpy(m.name, my_player.name, MP_FRIENDLY_NAME_LEN);
 
 	enet_socket_send(session_monitor, &service_provider, &b, 1);
+#endif
 }
 
 void MultiPlayer::send_poll_sessions()
 {
+#ifdef USE_ENET
 	ENetBuffer b;
 	MpMsgPollSessions m;
 
@@ -1196,10 +1186,12 @@ void MultiPlayer::send_poll_sessions()
 	misc.uuid_copy(m.login_id, service_login_id);
 
 	enet_socket_send(session_monitor, &service_provider, &b, 1);
+#endif
 }
 
 void MultiPlayer::send_req_session_id()
 {
+#ifdef USE_ENET
 	ENetBuffer b;
 	MpMsgReqSessionId m;
 
@@ -1215,10 +1207,12 @@ void MultiPlayer::send_req_session_id()
 	memcpy(m.session_password, joined_session.password, MP_FRIENDLY_NAME_LEN);
 
 	enet_socket_send(session_monitor, &service_provider, &b, 1);
+#endif
 }
 
 void MultiPlayer::send_req_session_addr()
 {
+#ifdef USE_ENET
 	ENetBuffer b;
 	MpMsgReqSessionAddr m;
 
@@ -1230,20 +1224,22 @@ void MultiPlayer::send_req_session_addr()
 
 	m.msg_id = MPMSG_REQ_SESSION_ADDR;
 	misc.uuid_copy(m.login_id, service_login_id);
-	misc.uuid_copy(m.session_id, joined_session.id);
+	misc.uuid_copy(m.session_id, joined_session.session_id);
 	memcpy(m.session_password, joined_session.password, MP_FRIENDLY_NAME_LEN);
 
 	enet_socket_send(session_monitor, &service_provider, &b, 1);
+#endif
 }
 
 void MultiPlayer::send_ping(ENetSocket s, ENetAddress *a)
 {
+#ifdef USE_ENET
 	ENetBuffer b;
 	MpMsgPing m;
 
 	if (!host)
 		return;
-	if (misc.uuid_is_null(joined_session.id))
+	if (misc.uuid_is_null(joined_session.session_id))
 		return;
 
 	b.data = &m;
@@ -1252,16 +1248,18 @@ void MultiPlayer::send_ping(ENetSocket s, ENetAddress *a)
 	m.msg_id = MPMSG_PING;
 
 	enet_socket_send(s, a, &b, 1);
+#endif
 }
 
 void MultiPlayer::send_req_host_nat_punch()
 {
+#ifdef USE_ENET
 	ENetBuffer b;
 	MpMsgReqHostNatPunch m;
 
 	if (!host)
 		return;
-	if (misc.uuid_is_null(joined_session.id))
+	if (misc.uuid_is_null(joined_session.session_id))
 		return;
 	if (misc.uuid_is_null(service_login_id))
 		return;
@@ -1271,22 +1269,26 @@ void MultiPlayer::send_req_host_nat_punch()
 
 	m.msg_id = MPMSG_REQ_HOST_NAT_PUNCH;
 	misc.uuid_copy(m.login_id, service_login_id);
-	misc.uuid_copy(m.session_id, joined_session.id);
+	misc.uuid_copy(m.session_id, joined_session.session_id);
 	memcpy(m.session_password, joined_session.password, MP_FRIENDLY_NAME_LEN);
 
 	enet_socket_send(host->socket, &service_provider, &b, 1);
+#endif
 }
 
 void MultiPlayer::send_service_ping()
 {
+#ifdef USE_ENET
 	if (session_monitor == ENET_SOCKET_NULL)
 		return;
 
 	send_ping(session_monitor, &service_provider);
+#endif
 }
 
 void MultiPlayer::do_host_nat_punch(MpMsgHostNatPunch *in)
 {
+#ifdef USE_ENET
 	ENetAddress a;
 	ENetBuffer b;
 	MpMsgPing m;
@@ -1298,6 +1300,7 @@ void MultiPlayer::do_host_nat_punch(MpMsgHostNatPunch *in)
 	a.port = in->port;
 
 	send_ping(host->socket, &a);
+#endif
 }
 
 // Returns pointer to the recv_buf when a message is received, with size set.
@@ -1313,6 +1316,7 @@ void MultiPlayer::do_host_nat_punch(MpMsgHostNatPunch *in)
 // Currently, pointers for from and size are always expected.
 char *MultiPlayer::receive(uint32_t *from, uint32_t *size, int *sysMsgCount)
 {
+#ifdef USE_ENET
 	int ret;
 	ENetEvent event;
 	PlayerDesc *player;
@@ -1339,13 +1343,8 @@ char *MultiPlayer::receive(uint32_t *from, uint32_t *size, int *sysMsgCount)
 
 	switch (event.type) {
 	case ENET_EVENT_TYPE_RECEIVE:
-		if (event.packet->dataLength < MP_RECV_BUFFER_SIZE) {
-			memcpy(recv_buf, event.packet->data,
-				event.packet->dataLength);
-			*size = event.packet->dataLength;
+		if (retrieve_packet(&event, size))
 			got_recv = recv_buf;
-		}
-		enet_packet_destroy(event.packet);
 
 		break;
 
@@ -1360,12 +1359,12 @@ char *MultiPlayer::receive(uint32_t *from, uint32_t *size, int *sysMsgCount)
 		}
 
 		if (!player) {
-			if (!(joined_session.flags & SESSION_PREGAME)) {
+			if (!(joined_session.flags & SessionFlags::Pregame)) {
 				enet_peer_disconnect(event.peer, 0);
 				break;
 			}
-			player = new PlayerDesc(&event.peer->address);
-			if (joined_session.flags & SESSION_HOSTING) {
+			player = new PlayerDesc("?Anonymous?", event.peer->address);
+			if (joined_session.flags & SessionFlags::Hosting) {
 				player->id = get_avail_player_id();
 			}
 		}
@@ -1386,7 +1385,7 @@ char *MultiPlayer::receive(uint32_t *from, uint32_t *size, int *sysMsgCount)
 
 		if (player) {
 			MSG("Player '%s' (%d) disconnected.\n", player->name, player->id);
-			if (joined_session.flags & SESSION_HOSTING) {
+			if (joined_session.flags & SessionFlags::Hosting) {
 				delete player;
 			} else {
 				// try to save in case of a reconnection or host acknowledgement
@@ -1405,6 +1404,8 @@ char *MultiPlayer::receive(uint32_t *from, uint32_t *size, int *sysMsgCount)
 	}
 
 	return got_recv;
+#endif
+	return NULL;
 }
 
 /*
@@ -1417,7 +1418,7 @@ static int sort_session_id(const void *a, const void *b)
 
 static int sort_session_name(const void *a, const void *b)
 {
-	return strcmp( ((SessionDesc *)a)->name_str(), ((SessionDesc *)b)->name_str() );
+	return strcmp( ((SessionDesc *)a)->session_name, ((SessionDesc *)b)->session_name );
 }
 
 // sort current_sessions
@@ -1439,3 +1440,25 @@ void MultiPlayer::sort_sessions(int sortType)
 		err_here();
 	}
 }
+
+int MultiPlayer::retrieve_packet(ENetEvent *event, uint32_t *size)
+{
+	int status = 0;
+#ifdef USE_ENET
+	if (event->packet->dataLength > recv_buffer_size) {
+		if (recv_buf) {
+			delete[] recv_buf;
+			recv_buf = new char[MP_RECV_MAX_BUFFER_SIZE];
+			recv_buffer_size = MP_RECV_MAX_BUFFER_SIZE;
+		}
+	}
+	if (event->packet->dataLength <= recv_buffer_size) {
+		memcpy(recv_buf, event->packet->data,
+			event->packet->dataLength);
+		*size = event->packet->dataLength;
+		status = 1;
+	}
+	enet_packet_destroy(event->packet);
+#endif
+	return status;
+}
